#!/usr/bin/env python
"""Common fixtures and utils for evc unit-tests."""
import copy

import pytest

from orion.core.evc import conflicts


@pytest.fixture
def parent_config():
    """Generate a new experiment configuration"""
    return dict(_id=1234, name="test", metadata={"user": "corneauf"}, version=1)


@pytest.fixture
def child_config(parent_config):
    """Generate a new experiment configuration"""
    config = copy.deepcopy(parent_config)
    config["_id"] = 1235
    config["refers"] = {"parent_id": 1234}
    config["version"] = 2

    return config


@pytest.fixture
def exp_no_child_conflict(storage, parent_config):
    """Generate an experiment name conflict"""
    storage.create_experiment(parent_config)
    return conflicts.ExperimentNameConflict(parent_config, parent_config)


@pytest.fixture
def exp_w_child_conflict(storage, parent_config, child_config):
    """Generate an experiment name conflict"""
    storage.create_experiment(parent_config)
    storage.create_experiment(child_config)
    return conflicts.ExperimentNameConflict(child_config, child_config)


@pytest.fixture
def exp_w_child_as_parent_conflict(storage, parent_config, child_config):
    """Generate an experiment name conflict"""
    storage.create_experiment(parent_config)
    storage.create_experiment(child_config)
    return conflicts.ExperimentNameConflict(parent_config, parent_config)


@pytest.fixture
def existing_exp_conflict(storage, parent_config):
    """Generate an experiment name conflict"""
    storage.create_experiment(parent_config)
    storage.create_experiment({"name": "dummy", "version": 1})
    return conflicts.ExperimentNameConflict(parent_config, parent_config)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Module running the set command
==============================

Update data of experiments and trials in the storage

"""
import argparse
import logging
import sys

import orion.core.io.experiment_builder as experiment_builder
from orion.core.utils.pptree import print_tree
from orion.core.utils.terminal import confirm_name
from orion.storage.base import get_storage

logger = logging.getLogger(__name__)


DESCRIPTION = """
Command to update trial attributes.

To change a trial status, simply give the experiment name,
trial id and status. (use `orion status --all` to get trial ids)
$ orion db set my-exp-name id=3cc91e851e13281ca2152c19d888e937 status=interrupted

To change all trials from a given status to another, simply give the two status
$ orion db set my-exp-name status=broken status=interrupted

Or `*` to apply the change to all trials
$ orion db set my-exp-name '*' status=interrupted

By default, trials of the last version of the experiment are selected.
Add --version to select a prior version. Note that the modification
is applied recursively to all child experiment, but not to the parents.
$ orion db set my-exp-name --version 1 status=broken status=interrupted
"""


CONFIRM_MESSAGE = """
Trials matching the query `{query}` for all experiments listed above
will be modified with `{update}`.
To select a specific version use --version <VERSION>.

Make sure to stop any worker currently executing one of these experiment.

To proceed, type again the name of the experiment: """


def add_subparser(parser):
    """Return the parser that needs to be used for this command"""
    set_parser = parser.add_parser(
        "set",
        description=DESCRIPTION,
        help="Update trials' attributes",
        formatter_class=argparse.RawTextHelpFormatter,
    )

    set_parser.set_defaults(func=main)

    set_parser.add_argument("name", help="Name of the experiment to delete.")

    set_parser.add_argument(
        "-c",
        "--config",
        type=argparse.FileType("r"),
        metavar="path-to-config",
        help="user provided orion configuration file",
    )

    set_parser.add_argument(
        "-v",
        "--version",
        type=int,
        default=None,
        help="specific version of experiment to fetch; "
        "(default: last version matching.)",
    )

    set_parser.add_argument(
        "-f",
        "--force",
        action="store_true",
        help="Force modify without asking to enter experiment name twice.",
    )

    set_parser.add_argument(
        "query",
        help=(
            f"Query for trials to update. Can be `*` to update all trials. "
            f"Otherwise format must be <attribute>=<value>. Ex: status=broken. "
            f"Supported attributes are {VALID_QUERY_ATTRS}"
        ),
    )

    set_parser.add_argument(
        "update",
        help=(
            f"Update for trials. Format must be <attribute>=<value>. "
            f"Ex: status=interrupted. "
            f"Supported attributes are {VALID_UPDATE_ATTRS}"
        ),
    )

    return set_parser


def process_updates(storage, root, query, update):
    """Update the matching trials of the given experiment and its children."""
    trials_total = 0
    for node in root:
        count = storage.update_trials(node.item, where=query, **update)
        logger.debug(
            "%d trials modified in experiment %s-v%d",
            count,
            node.item.name,
            node.item.version,
        )
        trials_total += count

    print(f"{trials_total} trials modified")


VALID_QUERY_ATTRS = ["status", "id"]
VALID_UPDATE_ATTRS = ["status"]


def build_query(query):
    """Convert query string to dict format

    String format must be <attr name>=<value>
    """
    if query.strip() == "*":
        return {}

    attribute, value = query.split("=")

    if attribute not in VALID_QUERY_ATTRS:
        raise ValueError(
            f"Invalid query attribute `{attribute}`. Must be one of {VALID_QUERY_ATTRS}"
        )

    if attribute == "id":
        attribute = "_id"

    return {attribute: value}


def build_update(update):
    """Convert update string to dict format

    String format must be <attr name>=<value>
    """
    attribute, value = update.split("=")

    if attribute not in VALID_UPDATE_ATTRS:
        raise ValueError(
            f"Invalid update attribute `{attribute}`. Must be one of {VALID_UPDATE_ATTRS}"
        )

    return {attribute: value}


def main(args):
    """Remove the experiment(s) or trial(s)."""
    config = experiment_builder.get_cmd_config(args)
    experiment_builder.setup_storage(config.get("storage"))

    # Find root experiment
    root = experiment_builder.load(
        name=args["name"], version=args.get("version", None)
    ).node

    try:
        query = build_query(args["query"])
        update = build_update(args["update"])
    except ValueError as e:
        print(f"Error: {e}", file=sys.stderr)
        return 1

    # List all experiments with children
    print_tree(root, nameattr="tree_name")

    confirmed = confirm_name(
        CONFIRM_MESSAGE.format(query=args["query"], update=args["update"]),
        args["name"],
        args["force"],
    )

    if not confirmed:
        print("Confirmation failed, aborting operation.")
        return 1

    storage = get_storage()

    process_updates(storage, root, query, update)

    return 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Installation script for Oríon."""
import os

from setuptools import setup

import versioneer


repo_root = os.path.dirname(os.path.abspath(__file__))


tests_require = ["pytest>=3.0.0", "scikit-learn"]


packages = [  # Packages must be sorted alphabetically to ease maintenance and merges.
    "orion.algo",
    "orion.analysis",
    "orion.benchmark",
    "orion.client",
    "orion.core",
    "orion.executor",
    "orion.plotting",
    "orion.serving",
    "orion.storage",
    "orion.testing",
]

setup_args = dict(
    name="orion",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description="Asynchronous [black-box] Optimization",
    long_description=open(
        os.path.join(repo_root, "README.rst"), "rt", encoding="utf8"
    ).read(),
    license="BSD-3-Clause",
    author=u"Epistímio",
    author_email="xavier.bouthillier@umontreal.ca",
    url="https://github.com/epistimio/orion",
    packages=packages,
    package_dir={"": "src"},
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "orion = orion.core.cli:main",
        ],
        "OptimizationAlgorithm": [
            "random = orion.algo.random:Random",
            "gridsearch = orion.algo.gridsearch:GridSearch",
            "asha = orion.algo.asha:ASHA",
            "hyperband = orion.algo.hyperband:Hyperband",
            "tpe = orion.algo.tpe:TPE",
            "EvolutionES = orion.algo.evolution_es:EvolutionES",
        ],
        "Storage": [
            "track = orion.storage.track:Track",
            "legacy = orion.storage.legacy:Legacy",
        ],
        "Executor": [
            "joblib = orion.executor.joblib_backend:Joblib",
            "dask = orion.executor.dask_backend:Dask",
        ],
    },
    install_requires=[
        "PyYAML",
        "pymongo>=3",
        "numpy",
        "scipy",
        "gitpython",
        "filelock",
        "tabulate",
        "AppDirs",
        "plotly",
        "kaleido",
        "requests",
        "pandas",
        "gunicorn",
        "falcon",
        "scikit-learn",
        "psutil",
        "joblib",
        "pytest>=3.0.0",
    ],
    tests_require=tests_require,
    setup_requires=["setuptools", "appdirs", "pytest-runner"],
    extras_require=dict(test=tests_require, dask=["dask[complete]"]),
    # "Zipped eggs don't play nicely with namespace packaging"
    # from https://github.com/pypa/sample-namespace-packages
    zip_safe=False,
)

setup_args["keywords"] = [
    "Machine Learning",
    "Deep Learning",
    "Distributed",
    "Optimization",
]

setup_args["platforms"] = ["Linux"]

setup_args["classifiers"] = [
    "Development Status :: 1 - Planning",
    "Intended Audience :: Developers",
    "Intended Audience :: Education",
    "Intended Audience :: Science/Research",
    "License :: OSI Approved :: BSD License",
    "Operating System :: POSIX",
    "Operating System :: Unix",
    "Programming Language :: Python",
    "Topic :: Scientific/Engineering",
    "Topic :: Scientific/Engineering :: Artificial Intelligence",
] + [("Programming Language :: Python :: %s" % x) for x in "3 3.6 3.7 3.8 3.9".split()]

if __name__ == "__main__":
    setup(**setup_args)



# -*- coding: utf-8 -*-
"""
Experiment Version Control System
=================================

Organize related experiments inside a tree-structure using adapters to maintain
compatibility between them.
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Collection of tests for :mod:`orion.storage`."""
import copy
import json
import logging
import os
import tempfile

import pytest

from orion.core.io.database import Database
from orion.core.io.database.pickleddb import PickledDB
from orion.core.utils.exceptions import MissingResultFile
from orion.core.utils.singleton import (
    SingletonAlreadyInstantiatedError,
    SingletonNotInstantiatedError,
    update_singletons,
)
from orion.core.worker.trial import Trial
from orion.storage.base import FailedUpdate
from orion.storage.legacy import get_database, setup_database
from orion.testing import OrionState

log = logging.getLogger(__name__)
log.setLevel(logging.WARNING)


base_experiment = {
    "name": "default_name",
    "version": 0,
    "metadata": {
        "user": "default_user",
        "user_script": "abc",
        "datetime": "2017-11-23T02:00:00",
    },
}

base_trial = {
    "experiment": "default_name",
    "status": "new",  # new, reserved, suspended, completed, broken
    "worker": None,
    "submit_time": "2017-11-23T02:00:00",
    "start_time": None,
    "end_time": None,
    "heartbeat": None,
    "results": [
        {"name": "loss", "type": "objective", "value": 2}  # objective, constraint
    ],
    "params": [
        {"name": "/encoding_layer", "type": "categorical", "value": "rnn"},
        {
            "name": "/decoding_layer",
            "type": "categorical",
            "value": "lstm_with_attention",
        },
    ],
}


mongodb_config = {
    "database": {
        "type": "MongoDB",
        "name": "orion_test",
        "username": "user",
        "password": "pass",
    }
}

db_backends = [{"type": "legacy", "database": mongodb_config}]


@pytest.mark.usefixtures("setup_pickleddb_database")
def test_setup_database_default(monkeypatch):
    """Test that database is setup using default config"""
    update_singletons()
    setup_database()
    database = Database()
    assert isinstance(database, PickledDB)


def test_setup_database_bad():
    """Test how setup fails when configuring with non-existant backends"""
    update_singletons()
    with pytest.raises(NotImplementedError) as exc:
        setup_database({"type": "idontexist"})

    assert exc.match("idontexist")


def test_setup_database_custom():
    """Test setup with local configuration"""
    update_singletons()
    setup_database({"type": "pickleddb", "host": "test.pkl"})
    database = Database()
    assert isinstance(database, PickledDB)
    assert database.host == os.path.abspath("test.pkl")


def test_setup_database_bad_override():
    """Test setup with different type than existing singleton"""
    update_singletons()
    setup_database({"type": "pickleddb", "host": "test.pkl"})
    database = Database()
    assert isinstance(database, PickledDB)
    with pytest.raises(SingletonAlreadyInstantiatedError) as exc:
        setup_database({"type": "mongodb"})

    assert exc.match("A singleton instance of \(type: Database\)")


@pytest.mark.xfail(reason="Fix this when introducing #135 in v0.2.0")
def test_setup_database_bad_config_override():
    """Test setup with different config than existing singleton"""
    update_singletons()
    setup_database({"type": "pickleddb", "host": "test.pkl"})
    database = Database()
    assert isinstance(database, PickledDB)
    with pytest.raises(SingletonAlreadyInstantiatedError):
        setup_database({"type": "pickleddb", "host": "other.pkl"})


def test_get_database_uninitiated():
    """Test that get database fails if no database singleton exist"""
    update_singletons()
    with pytest.raises(SingletonNotInstantiatedError) as exc:
        get_database()

    assert exc.match("No singleton instance of \(type: Database\) was created")


def test_get_database():
    """Test that get database gets the singleton"""
    update_singletons()
    setup_database({"type": "pickleddb", "host": "test.pkl"})
    database = get_database()
    assert isinstance(database, PickledDB)
    assert get_database() == database


class TestLegacyStorage:
    """Test Legacy Storage retrieve result mechanic separately"""

    def test_push_trial_results(self, storage=None):
        """Successfully push a completed trial into database."""
        reserved_trial = copy.deepcopy(base_trial)
        reserved_trial["status"] = "reserved"
        with OrionState(
            experiments=[], trials=[reserved_trial], storage=storage
        ) as cfg:
            storage = cfg.storage()
            trial = storage.get_trial(Trial(**reserved_trial))
            results = [Trial.Result(name="loss", type="objective", value=2)]
            trial.results = results
            assert storage.push_trial_results(trial), "should update successfully"

            trial2 = storage.get_trial(trial)
            assert trial2.results == results

    def test_push_trial_results_unreserved(self, storage=None):
        """Successfully push a completed trial into database."""
        with OrionState(experiments=[], trials=[base_trial], storage=storage) as cfg:
            storage = cfg.storage()
            trial = storage.get_trial(Trial(**base_trial))
            results = [Trial.Result(name="loss", type="objective", value=2)]
            trial.results = results
            with pytest.raises(FailedUpdate):
                storage.push_trial_results(trial)



# -*- coding: utf-8 -*-
"""
Provide tools to calculate regret
=================================
"""
import numpy
import pandas as pd


def regret(trials, names=("best", "best_id")):
    """
    Calculates the regret for a collection of :class:`orion.core.worker.trial.Trial`. The regret is
    calculated sequentially from the order of the collection.

    Parameters
    ----------
    trials: DataFrame or dict
        A dataframe of trials containing, at least, the columns 'objective' and 'id'. Or a dict
        equivalent.
    names:
        A tuple containing the names of the columns. Default is ('best', 'best-id').

    Returns
    -------
    A copy of the original dataframe with two new columns containing respectively the best value
    so far and its trial id.
    """
    if len(names) != 2:
        raise ValueError(
            f"`names` requires a tuple with 2 elements. {len(names)} provided."
        )

    df = pd.DataFrame(trials, copy=True)
    if df.empty:
        return df

    regrets_idx = get_regrets_idx(df["objective"])
    df[names[0]] = df["objective"].to_numpy()[list(regrets_idx)]
    df[names[1]] = df["id"].to_numpy()[regrets_idx]

    return df


def get_regrets_idx(objectives):
    """Return the indices corresponding to the cumulative minimum"""
    minima = numpy.minimum.accumulate(objectives)
    diff = numpy.diff(minima)
    jumps = numpy.arange(len(objectives))
    jumps[1:] *= diff != 0
    jumps = numpy.maximum.accumulate(jumps)
    return jumps



"""Perform tests for the REST endpoint `/plots`"""
import pytest

from orion.testing import create_experiment

config = dict(
    name="experiment-name",
    space={"x": "uniform(0, 200)"},
    metadata={
        "user": "test-user",
        "orion_version": "XYZ",
        "VCS": {
            "type": "git",
            "is_dirty": False,
            "HEAD_sha": "test",
            "active_branch": None,
            "diff_sha": "diff",
        },
    },
    version=1,
    pool_size=1,
    max_trials=10,
    working_dir="",
    algorithms={"random": {"seed": 1}},
    producer={"strategy": "NoParallelStrategy"},
)

trial_config = {
    "experiment": 0,
    "status": "completed",
    "worker": None,
    "start_time": None,
    "end_time": None,
    "heartbeat": None,
    "results": [],
    "params": [],
}


def test_root_not_available(client):
    """Tests that plots/regret is not available"""
    response = client.simulate_get("/plots/regret")

    assert response.status == "404 Not Found"


@pytest.mark.usefixtures("version_XYZ")
class TestRegretPlots:
    """Tests regret plots"""

    def test_unknown_experiment(self, client):
        """Tests that the API returns a 404 Not Found when an unknown experiment is queried."""
        response = client.simulate_get("/plots/regret/unknown-experiment")

        assert response.status == "404 Not Found"
        assert response.json == {
            "title": "Experiment not found",
            "description": 'Experiment "unknown-experiment" does not exist',
        }

    def test_plot(self, client):
        """Tests that the API returns the plot in json format."""
        with create_experiment(config, trial_config, ["completed"]) as (
            _,
            _,
            experiment,
        ):
            response = client.simulate_get("/plots/regret/experiment-name")

        assert response.status == "200 OK"
        assert response.json
        assert list(response.json.keys()) == ["data", "layout"]

    def test_no_trials(self, client):
        """Tests that the API returns an empty figure when no trials are found."""
        with create_experiment(config, trial_config, []) as (
            _,
            _,
            experiment,
        ):
            response = client.simulate_get("/plots/regret/experiment-name")

        assert response.status == "200 OK"
        assert list(response.json.keys()) == ["data", "layout"]


@pytest.mark.usefixtures("version_XYZ")
class TestParallelCoordinatesPlots:
    """Tests parallel coordinates plots"""

    def test_unknown_experiment(self, client):
        """Tests that the API returns a 404 Not Found when an unknown experiment is queried."""
        response = client.simulate_get("/plots/parallel_coordinates/unknown-experiment")

        assert response.status == "404 Not Found"
        assert response.json == {
            "title": "Experiment not found",
            "description": 'Experiment "unknown-experiment" does not exist',
        }

    def test_plot(self, client):
        """Tests that the API returns the plot in json format."""
        with create_experiment(config, trial_config, ["completed"]) as (
            _,
            _,
            experiment,
        ):
            response = client.simulate_get(
                "/plots/parallel_coordinates/experiment-name"
            )

        assert response.status == "200 OK"
        assert response.json
        assert list(response.json.keys()) == ["data", "layout"]

    def test_no_trials(self, client):
        """Tests that the API returns an empty figure when no trials are found."""
        with create_experiment(config, trial_config, []) as (
            _,
            _,
            experiment,
        ):
            response = client.simulate_get(
                "/plots/parallel_coordinates/experiment-name"
            )

        assert response.status == "200 OK"
        assert list(response.json.keys()) == ["data", "layout"]


@pytest.mark.usefixtures("version_XYZ")
class TestPartialDependenciesPlots:
    """Tests parallel coordinates plots"""

    def test_unknown_experiment(self, client):
        """Tests that the API returns a 404 Not Found when an unknown experiment is queried."""
        response = client.simulate_get("/plots/partial_dependencies/unknown-experiment")

        assert response.status == "404 Not Found"
        assert response.json == {
            "title": "Experiment not found",
            "description": 'Experiment "unknown-experiment" does not exist',
        }

    def test_plot(self, client):
        """Tests that the API returns the plot in json format."""
        with create_experiment(config, trial_config, ["completed"]) as (
            _,
            _,
            experiment,
        ):
            response = client.simulate_get(
                "/plots/partial_dependencies/experiment-name"
            )

        assert response.status == "200 OK"
        assert response.json
        assert list(response.json.keys()) == ["data", "layout"]

    def test_no_trials(self, client):
        """Tests that the API returns an empty figure when no trials are found."""
        with create_experiment(config, trial_config, []) as (
            _,
            _,
            experiment,
        ):
            response = client.simulate_get(
                "/plots/partial_dependencies/experiment-name"
            )

        assert response.status == "200 OK"
        assert list(response.json.keys()) == ["data", "layout"]


@pytest.mark.usefixtures("version_XYZ")
class TestLPIPlots:
    """Tests lpi plots"""

    def test_unknown_experiment(self, client):
        """Tests that the API returns a 404 Not Found when an unknown experiment is queried."""
        response = client.simulate_get("/plots/lpi/unknown-experiment")

        assert response.status == "404 Not Found"
        assert response.json == {
            "title": "Experiment not found",
            "description": 'Experiment "unknown-experiment" does not exist',
        }

    def test_plot(self, client):
        """Tests that the API returns the plot in json format."""
        with create_experiment(config, trial_config, ["completed"]) as (
            _,
            _,
            experiment,
        ):
            response = client.simulate_get("/plots/lpi/experiment-name")

        assert response.status == "200 OK"
        assert response.json
        assert list(response.json.keys()) == ["data", "layout"]

    def test_no_trials(self, client):
        """Tests that the API returns an empty figure when no trials are found."""
        with create_experiment(config, trial_config, []) as (
            _,
            _,
            experiment,
        ):
            response = client.simulate_get("/plots/lpi/experiment-name")

        assert response.status == "200 OK"
        assert list(response.json.keys()) == ["data", "layout"]



# -*- coding: utf-8 -*-
"""Tests :func:`orion.analysis.regret`"""
import pandas as pd
import pytest

from orion.analysis import regret


def test_accept_dict():
    """Tests dictionary parameter is supported"""
    data = {"id": ["a", "b", "c", "d"], "objective": [0.1, 0.2, 0.3, 0.5]}

    result = regret(data)

    assert type(result) is pd.DataFrame


def test_accept_empty():
    """Tests an empty dataframe is returned if you give an empty dataframe"""
    empty_frame = pd.DataFrame()
    result = regret(empty_frame)

    assert result.empty
    assert result.equals(empty_frame)

    empty_frame = pd.DataFrame(columns=["id", "objective"])
    result = regret(empty_frame)

    assert result.empty
    assert "best" not in result.columns
    assert "best_id" not in result.columns
    assert result.equals(empty_frame)


def test_parameter_not_modified():
    """Tests the original dataframe is not modified"""
    data = pd.DataFrame(
        data={"id": ["a", "b", "c", "d"], "objective": [0.1, 0.2, 0.3, 0.5]}
    )

    result = regret(data)

    assert len(data.columns) == 2
    assert len(result.columns) == 4


def test_length_names():
    """Tests the length of `names` parameter is two"""
    data = pd.DataFrame(data={"id": ["a", "b"], "objective": [0.1, 0.2]})

    with pytest.raises(ValueError) as exception:
        regret(data, names=())
    assert "`names` requires a tuple with 2 elements. 0 provided." in str(
        exception.value
    )

    with pytest.raises(ValueError) as exception:
        regret(data, names=("a", "b", "c"))
    assert "`names` requires a tuple with 2 elements. 3 provided." in str(
        exception.value
    )


def test_default_column_names():
    """Tests default column names"""
    data = pd.DataFrame(
        data={"id": ["a", "b", "c", "d"], "objective": [0.1, 0.2, 0.3, 0.5]}
    )

    result = regret(data)

    assert "best" in result.columns
    assert "best_id" in result.columns


def test_input_column_names():
    """Tests custom column names"""
    data = pd.DataFrame(
        data={"id": ["a", "b", "c", "d"], "objective": [0.1, 0.2, 0.3, 0.5]}
    )

    result = regret(data, names=("something", "else"))

    assert "something" in result.columns
    assert "else" in result.columns


def test_regret_sequential():
    """Tests that cumulative best objectives are linked to their respective ids"""
    data = pd.DataFrame(
        data={
            "id": ["a", "b", "c", "d"],
            "objective": [10, 12, 8, 9],
        }
    )

    expected_best = [10, 10, 8, 8]
    expected_ids = ["a", "a", "c", "c"]

    result = regret(data)

    assert all(result["best"].values == expected_best)
    assert all(result["best_id"].values == expected_ids)

    data = pd.DataFrame(
        data={
            "id": ["a", "b", "c"],
            "objective": [10, 9, 8],
        }
    )

    expected_best = [10, 9, 8]
    expected_ids = ["a", "b", "c"]

    result = regret(data)

    assert all(result["best"].values == expected_best)
    assert all(result["best_id"].values == expected_ids)

    data = pd.DataFrame(
        data={
            "id": ["a", "b", "c"],
            "objective": [8, 9, 10],
        }
    )

    expected_best = [8, 8, 8]
    expected_ids = ["a", "a", "a"]

    result = regret(data)

    assert all(result["best"].values == expected_best)
    assert all(result["best_id"].values == expected_ids)

    data = pd.DataFrame(
        data={
            "id": ["a"],
            "objective": [10],
        }
    )

    result = regret(data)

    assert all(result["best"].values == [10])
    assert all(result["best_id"].values == ["a"])


def test_regret_equal():
    """Tests instances where trials share best objectives"""
    data = pd.DataFrame(
        data={
            "id": ["a", "b", "c"],
            "objective": [8, 9, 8],
        }
    )

    expected_best = [8, 8, 8]
    expected_ids = ["a", "a", "a"]

    result = regret(data)

    assert all(result["best"].values == expected_best)
    assert all(result["best_id"].values == expected_ids)

    data = pd.DataFrame(
        data={
            "id": ["a", "b", "c"],
            "objective": [8, 8, 8],
        }
    )

    expected_best = [8, 8, 8]
    expected_ids = ["a", "a", "a"]

    result = regret(data)

    assert all(result["best"].values == expected_best)
    assert all(result["best_id"].values == expected_ids)



# -*- coding: utf-8 -*-
"""
Represent the '/' REST endpoint
===============================
"""
import json

import orion.core
from orion.storage.base import get_storage


class RuntimeResource(object):
    """Handle requests for the '/' REST endpoint"""

    def __init__(self):
        pass

    def on_get(self, req, resp):
        """Handle the HTTP GET requests for the '/' endpoint

        Parameters
        ----------
        req
            The request
        resp
            The response to send back
        """
        database = get_storage()._db.__class__.__name__
        response = {
            "orion": orion.core.__version__,
            "server": "gunicorn",
            "database": database,
        }

        resp.body = json.dumps(response, indent=4)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Collection of tests for :mod:`orion.core.worker.consumer`."""
import os
import signal
import subprocess
import tempfile
import time

import pytest

import orion.core.io.experiment_builder as experiment_builder
import orion.core.io.resolve_config as resolve_config
import orion.core.utils.backward as backward
import orion.core.worker.consumer as consumer
from orion.core.utils.exceptions import BranchingEvent, MissingResultFile
from orion.core.utils.format_trials import tuple_to_trial

Consumer = consumer.Consumer


@pytest.fixture
def config(exp_config):
    """Return a configuration."""
    config = exp_config[0][0]
    config["metadata"]["user_args"] = ["--x~uniform(-50, 50)"]
    config["metadata"]["VCS"] = resolve_config.infer_versioning_metadata(
        config["metadata"]["user_script"]
    )
    config["name"] = "exp"
    config["working_dir"] = "/tmp/orion"
    backward.populate_space(config)
    config["space"] = config["metadata"]["priors"]
    return config


@pytest.mark.usefixtures("storage")
def test_trials_interrupted_sigterm(config, monkeypatch):
    """Check if a trial is set as interrupted when a signal is raised."""

    def mock_popen(self, *args, **kwargs):
        os.kill(os.getpid(), signal.SIGTERM)

    exp = experiment_builder.build(**config)

    monkeypatch.setattr(subprocess.Popen, "wait", mock_popen)

    trial = tuple_to_trial((1.0,), exp.space)

    con = Consumer(exp)

    with pytest.raises(KeyboardInterrupt):
        con(trial)


@pytest.mark.usefixtures("storage")
def test_trial_working_dir_is_changed(config):
    """Check that trial has its working_dir attribute changed."""
    exp = experiment_builder.build(**config)

    trial = tuple_to_trial((1.0,), exp.space)

    exp.register_trial(trial, status="reserved")

    con = Consumer(exp)
    con(trial)

    assert trial.working_dir is not None
    assert trial.working_dir == con.working_dir + "/exp_" + trial.id


@pytest.mark.usefixtures("storage")
def test_code_changed(config, monkeypatch):
    """Check that trial has its working_dir attribute changed."""
    exp = experiment_builder.build(**config)

    trial = tuple_to_trial((1.0,), exp.space)

    exp.register_trial(trial, status="reserved")

    con = Consumer(exp)

    def code_changed(user_script):
        return dict(
            type="git",
            is_dirty=True,
            HEAD_sha="changed",
            active_branch="new_branch",
            diff_sha="new_diff",
        )

    monkeypatch.setattr(consumer, "infer_versioning_metadata", code_changed)

    with pytest.raises(BranchingEvent) as exc:
        con(trial)

    assert exc.match("Code changed between execution of 2 trials")


@pytest.mark.usefixtures("storage")
def test_retrieve_result_nofile(config):
    """Test retrieve result"""
    results_file = tempfile.NamedTemporaryFile(
        mode="w", prefix="results_", suffix=".log", dir=".", delete=True
    )

    exp = experiment_builder.build(**config)

    con = Consumer(exp)

    with pytest.raises(MissingResultFile) as exec:
        con.retrieve_results(results_file)

    results_file.close()

    assert exec.match(r"Cannot parse result file")



"""
===========
How to save
===========


Image
-----

Plots can be saved to multiple image formats using ``plotly``'s 
`fig.write_image() <https://plotly.github.io/plotly.py-docs/generated/plotly.io.write_image.html>`_.

"""

from orion.client import get_experiment

# Specify the database where the experiments are stored. We use a local PickleDB here.
storage = dict(type="legacy", database=dict(type="pickleddb", host="../../db.pkl"))

# Load the data for the specified experiment
experiment = get_experiment("2-dim-exp", storage=storage)
fig = experiment.plot.regret()
fig.write_image("regret.png")

#%%
# See `plotly`'s `short tutorial <https://plotly.com/python/static-image-export/>`_
# for more examples.
#
# HTML
# ----
#
# Using ``write_image`` is convenient for static images that can be included in PDF articles
# but it looses all advantages of plotly's interactive plots.
# You can save an HTML version instead using
# `fig.write_html() <https://plotly.github.io/plotly.py-docs/generated/plotly.io.write_html.html>`_
# if you wish to keep an interactive version of the plot.

fig.write_html("regret.html")

#%%
# JSON
# ----
#
# Plots can also be saved to json using
# `fig.to_json() <https://plotly.github.io/plotly.py-docs/generated/plotly.io.to_json.html>`_.
# This is the format used by the :ref:`web-api` for instance.

fig.to_json("regret.json")

# sphinx_gallery_thumbnail_path = '_static/save.png'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Perform functional tests for the parsing of the `hunt` command."""
import argparse
import os

import pytest

from orion.core.cli import hunt


def _create_parser(need_subparser=True):
    parser = argparse.ArgumentParser()

    if need_subparser:
        subparsers = parser.add_subparsers()
        return parser, subparsers

    return parser


def test_hunt_command_full_parsing(setup_pickleddb_database, monkeypatch):
    """Test the parsing of the `hunt` command"""
    monkeypatch.chdir(os.path.dirname(os.path.abspath(__file__)))
    parser, subparsers = _create_parser()
    args_list = [
        "hunt",
        "-n",
        "test",
        "--config",
        "./orion_config_random.yaml",
        "--max-trials",
        "400",
        "--pool-size",
        "4",
        "--worker-trials",
        "5",
        "./black_box.py",
        "-x~normal(1,1)",
    ]

    hunt.add_subparser(subparsers)
    subparsers.choices["hunt"].set_defaults(func="")

    args = vars(parser.parse_args(args_list))
    assert args["name"] == "test"
    assert args["config"].name == "./orion_config_random.yaml"
    assert args["user_args"] == ["./black_box.py", "-x~normal(1,1)"]
    assert args["pool_size"] == 4
    assert args["max_trials"] == 400
    assert args["worker_trials"] == 5



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for :mod:`orion.benchmark.benchmark_client`."""
import copy
import logging

import pytest

import orion.benchmark.benchmark_client as benchmark_client
import orion.core
from orion.benchmark.assessment import AverageResult
from orion.benchmark.benchmark_client import get_or_create_benchmark
from orion.benchmark.task import CarromTable, RosenBrock
from orion.core.io.database.ephemeraldb import EphemeralDB
from orion.core.io.database.pickleddb import PickledDB
from orion.core.utils.exceptions import NoConfigurationError
from orion.core.utils.singleton import SingletonNotInstantiatedError, update_singletons
from orion.storage.base import get_storage
from orion.storage.legacy import Legacy
from orion.testing.state import OrionState


class DummyTask:
    """Dummy invalid benchmark task"""

    pass


class DummyAssess:
    """Dummy invalid benchmark assessment"""

    pass


def count_benchmarks():
    """Count experiments in storage"""
    return len(get_storage().fetch_benchmark({}))


class TestCreateBenchmark:
    """Test Benchmark creation"""

    @pytest.mark.usefixtures("setup_pickleddb_database")
    def test_create_benchmark_no_storage(self, benchmark_config_py):
        """Test creation if storage is not configured"""
        name = "oopsie_forgot_a_storage"
        host = orion.core.config.storage.database.host

        with OrionState(storage=orion.core.config.storage.to_dict()) as cfg:
            # Reset the Storage and drop instances so that get_storage() would fail.
            cfg.cleanup()
            cfg.singletons = update_singletons()

            # Make sure storage must be instantiated during `get_or_create_benchmark()`
            with pytest.raises(SingletonNotInstantiatedError):
                get_storage()

            get_or_create_benchmark(**benchmark_config_py)

            storage = get_storage()

            assert isinstance(storage, Legacy)
            assert isinstance(storage._db, PickledDB)
            assert storage._db.host == host

    def test_create_benchmark_with_storage(self, benchmark_config_py):
        """Test benchmark instance has the storage configurations"""

        config = copy.deepcopy(benchmark_config_py)
        storage = {"type": "legacy", "database": {"type": "EphemeralDB"}}
        with OrionState(storage=storage):
            config["storage"] = storage
            bm = get_or_create_benchmark(**config)

            assert bm.storage_config == config["storage"]

    def test_create_benchmark_bad_storage(self, benchmark_config_py):
        """Test error message if storage is not configured properly"""
        name = "oopsie_bad_storage"
        # Make sure there is no existing storage singleton
        update_singletons()

        with pytest.raises(NotImplementedError) as exc:
            benchmark_config_py["storage"] = {
                "type": "legacy",
                "database": {"type": "idontexist"},
            }
            get_or_create_benchmark(**benchmark_config_py)

        assert (
            "Could not find implementation of AbstractDB, type = 'idontexist'"
            in str(exc.value)
        )

    def test_create_experiment_debug_mode(self, tmp_path, benchmark_config_py):
        """Test that EphemeralDB is used in debug mode whatever the storage config given"""
        update_singletons()

        conf_file = str(tmp_path / "db.pkl")

        config = copy.deepcopy(benchmark_config_py)
        config["storage"] = {
            "type": "legacy",
            "database": {"type": "pickleddb", "host": conf_file},
        }

        get_or_create_benchmark(**config)

        storage = get_storage()

        assert isinstance(storage, Legacy)
        assert isinstance(storage._db, PickledDB)

        update_singletons()
        config["storage"] = {"type": "legacy", "database": {"type": "pickleddb"}}
        config["debug"] = True
        get_or_create_benchmark(**config)

        storage = get_storage()

        assert isinstance(storage, Legacy)
        assert isinstance(storage._db, EphemeralDB)

    def test_create_benchmark(self, benchmark_config, benchmark_config_py):
        """Test creation with valid configuration"""
        with OrionState():
            bm1 = get_or_create_benchmark(**benchmark_config_py)

            bm2 = get_or_create_benchmark("bm00001")

            assert bm1.configuration == benchmark_config

            assert bm1.configuration == bm2.configuration

    def test_create_with_only_name(self):
        """Test creation with a non-existing benchmark name"""
        with OrionState():
            name = "bm00001"
            with pytest.raises(NoConfigurationError) as exc:
                get_or_create_benchmark(name)

            assert "Benchmark {} does not exist in DB".format(name) in str(exc.value)

    def test_create_with_different_configure(self, benchmark_config_py, caplog):
        """Test creation with same name but different configure"""
        with OrionState():
            config = copy.deepcopy(benchmark_config_py)
            bm1 = get_or_create_benchmark(**config)

            config = copy.deepcopy(benchmark_config_py)
            config["targets"][0]["assess"] = [AverageResult(2)]

            with caplog.at_level(
                logging.WARNING, logger="orion.benchmark.benchmark_client"
            ):
                bm2 = get_or_create_benchmark(**config)

            assert bm2.configuration == bm1.configuration
            assert (
                "Benchmark with same name is found but has different configuration, "
                "which will be used for this creation." in caplog.text
            )

            caplog.clear()
            config = copy.deepcopy(benchmark_config_py)
            config["targets"][0]["task"] = [RosenBrock(26, dim=3), CarromTable(20)]
            with caplog.at_level(
                logging.WARNING, logger="orion.benchmark.benchmark_client"
            ):
                bm3 = get_or_create_benchmark(**config)

            assert bm3.configuration == bm1.configuration
            assert (
                "Benchmark with same name is found but has different configuration, "
                "which will be used for this creation." in caplog.text
            )

    def test_create_with_invalid_algorithms(self, benchmark_config_py):
        """Test creation with a not existed algorithm"""
        with OrionState():

            with pytest.raises(NotImplementedError) as exc:
                benchmark_config_py["algorithms"] = [
                    {"algorithm": {"fake_algorithm": {"seed": 1}}}
                ]
                get_or_create_benchmark(**benchmark_config_py)
            assert "Could not find implementation of BaseAlgorithm" in str(exc.value)

    def test_create_with_deterministic_algorithm(self, benchmark_config_py):
        algorithms = [
            {"algorithm": {"random": {"seed": 1}}},
            {"algorithm": {"gridsearch": {"n_values": 50}}, "deterministic": True},
        ]
        with OrionState():
            config = copy.deepcopy(benchmark_config_py)
            config["algorithms"] = algorithms
            bm = get_or_create_benchmark(**config)

            for study in bm.studies:
                for status in study.status():
                    algo = status["algorithm"]
                    if algo == "gridsearch":
                        assert status["experiments"] == 1
                    else:
                        assert status["experiments"] == study.assessment.task_num

    def test_create_with_invalid_targets(self, benchmark_config_py):
        """Test creation with invalid Task and Assessment"""
        with OrionState():

            with pytest.raises(AttributeError) as exc:
                config = copy.deepcopy(benchmark_config_py)
                config["targets"] = [
                    {"assess": [AverageResult(2)], "task": [DummyTask]}
                ]
                get_or_create_benchmark(**config)

            assert "type object '{}' has no attribute ".format("DummyTask") in str(
                exc.value
            )

            with pytest.raises(AttributeError) as exc:
                config = copy.deepcopy(benchmark_config_py)
                config["targets"] = [
                    {"assess": [DummyAssess], "task": [RosenBrock(25, dim=3)]}
                ]
                get_or_create_benchmark(**config)

            assert "type object '{}' has no attribute ".format("DummyAssess") in str(
                exc.value
            )

    def test_create_with_not_loaded_targets(self, benchmark_config):
        """Test creation with assessment or task does not exist or not loaded"""

        cfg_invalid_assess = copy.deepcopy(benchmark_config)
        cfg_invalid_assess["targets"][0]["assess"]["idontexist"] = {"task_num": 2}

        with OrionState(benchmarks=cfg_invalid_assess):
            with pytest.raises(NotImplementedError) as exc:
                get_or_create_benchmark(benchmark_config["name"])
            assert "Could not find implementation of BaseAssess" in str(exc.value)

        cfg_invalid_task = copy.deepcopy(benchmark_config)
        cfg_invalid_task["targets"][0]["task"]["idontexist"] = {"max_trials": 2}

        with OrionState(benchmarks=cfg_invalid_task):
            with pytest.raises(NotImplementedError) as exc:
                get_or_create_benchmark(benchmark_config["name"])
            assert "Could not find implementation of BaseTask" in str(exc.value)

    def test_create_with_not_exist_targets_parameters(self, benchmark_config):
        """Test creation with not existing assessment parameters"""

        benchmark_config["targets"][0]["assess"]["AverageResult"] = {
            "task_num": 2,
            "idontexist": 100,
        }

        with OrionState(benchmarks=benchmark_config):
            with pytest.raises(TypeError) as exc:
                get_or_create_benchmark(benchmark_config["name"])
            assert "__init__() got an unexpected keyword argument 'idontexist'" in str(
                exc.value
            )

    def test_create_from_db_config(self, benchmark_config):
        """Test creation from existing db configubenchmark_configre"""
        with OrionState(benchmarks=copy.deepcopy(benchmark_config)):
            bm = get_or_create_benchmark(benchmark_config["name"])
            assert bm.configuration == benchmark_config

    def test_create_race_condition(
        self, benchmark_config, benchmark_config_py, monkeypatch, caplog
    ):
        """Test creation in race condition"""
        with OrionState(benchmarks=benchmark_config):

            def insert_race_condition(*args, **kwargs):
                if insert_race_condition.count == 0:
                    data = {}
                else:
                    data = benchmark_config

                insert_race_condition.count += 1

                return data

            insert_race_condition.count = 0
            monkeypatch.setattr(
                benchmark_client, "_fetch_benchmark", insert_race_condition
            )

            with caplog.at_level(
                logging.INFO, logger="orion.benchmark.benchmark_client"
            ):
                bm = benchmark_client.get_or_create_benchmark(**benchmark_config_py)

            assert (
                "Benchmark registration failed. This is likely due to a race condition. "
                "Now rolling back and re-attempting building it." in caplog.text
            )
            assert insert_race_condition.count == 2

            del benchmark_config["_id"]

            assert bm.configuration == benchmark_config
            assert count_benchmarks() == 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Module to list experiments
==========================

List experiments in terminal

"""
import logging

import orion.core.io.experiment_builder as experiment_builder
from orion.core.cli import base as cli
from orion.core.utils.pptree import print_tree
from orion.storage.base import get_storage

log = logging.getLogger(__name__)
SHORT_DESCRIPTION = "Gives a list of experiments"
DESCRIPTION = """
This command gives a list of your experiments in a easy-to-read tree-like structure.
"""


def add_subparser(parser):
    """Add the subparser that needs to be used for this command"""
    list_parser = parser.add_parser(
        "list", help=SHORT_DESCRIPTION, description=DESCRIPTION
    )

    cli.get_basic_args_group(list_parser)

    list_parser.set_defaults(func=main)

    return list_parser


def main(args):
    """List all experiments inside database."""
    config = experiment_builder.get_cmd_config(args)
    experiment_builder.setup_storage(config.get("storage"))

    query = {}

    if args["name"]:
        query["name"] = args["name"]

    experiments = get_storage().fetch_experiments(query)

    if args["name"]:
        root_experiments = experiments
    else:
        root_experiments = [
            exp
            for exp in experiments
            if exp["refers"].get("root_id", exp["_id"]) == exp["_id"]
        ]

    if not root_experiments:
        print("No experiment found")
        return

    for root_experiment in root_experiments:
        root = experiment_builder.load(
            name=root_experiment["name"], version=root_experiment.get("version")
        ).node
        print_tree(root, nameattr="tree_name")



"""
====================
Partial Dependencies
====================

.. hint::

   Conveys a broad overview of the search space and what has been explored during the experiment.
   Helps identifying best optimal regions of the space.

The partial dependency computes the average predicted performance
with respect to a set of hyperparameters, marginalizing out the other hyperparameters.

To predict the performance on unseen set of hyperparameters, we train a regression model on
available trial history. We build a grid of `g` points for a hyperparameter of interest, or a 2-D
grid of `g ^ 2` points for a pair of hyperparameters.  We sample a group of `n` set of
hyperparameters from the entire space to marginalize over the other hyperparameters. For each value
of the grid, we compute the prediction
of the regression model on all points of the group, with the hyperparameter of interest set to a
value of the grid. For instance, for a 1-D grid of `g` points and a group of `n` points, we compute
`g * n` predictions.

For a search space of `d` hyperparameters, the partial dependency plot is organized as a matrix of
`(d, d)` subplots. The subplots on the diagonal show the partial dependency of each hyperparameters
separately, while the subplots below the diagonal show the partial dependency between two
hyperparameters. Let's look at a simple example to make it more concrete.

.. autofunction:: orion.plotting.base.partial_dependencies
    :noindex:

The partial dependencies plot can be executed directly from the ``experiment`` with
``plot.partial_dependencies()`` as shown in the example below.

"""
from orion.client import get_experiment

# Specify the database where the experiments are stored. We use a local PickleDB here.
storage = dict(type="legacy", database=dict(type="pickleddb", host="../db.pkl"))

# Load the data for the specified experiment
experiment = get_experiment("2-dim-exp", storage=storage)
fig = experiment.plot.partial_dependencies()
fig

#%%
# For the plots on the diagonal, the y-axis is the objective and the x-axis is the value of the
# corresponding hyperparameter. For the contour plots below the diagonal, the y-axis and x-axis are
# the values of the corresponding hyperparameters labelled on the left and at the bottom. The
# objective is represented as a color gradient in the contour plots. The light blue area in
# the plots on the diagonal represents the standard deviation of the predicted objective when
# varying the other hyperparameters over the search space.
# The black dots represents the
# trials in the current history of the experiment. If you hover your cursor over one dot, you will
# see the configuration of the corresponding trial following this format:
#
# ::
#
#    ID: <trial id>
#    value: <objective>
#    time: <completed time>
#    parameters
#      <name>: <value>
#
# Even for a simple 2-d search space, the partial dependency is very useful. We see very cleary in this example
# the optimal regions for both hyperparameters and we can see as well that the optimal region for
# learning rates is larger when the dropout is low, and narrower when dropout approaches 0.5.
#
# .. TODO::
#
#     Make one toy example where two HPs are dependent.
#

#%%
# Options
# -------
#
# Params
# ~~~~~~
#
# The simple example involved only 2 hyperparameters, but typical search spaces can be much larger.
# The partial dependency plot becomes hard to read with more than 3-5 hyperparameters dependency on
# the size of your screen. With a fix width like in this documentation, 5 hyperparameters are
# impossible to read as you can see below.
# (Data coming from tutorial :ref:`sphx_glr_auto_tutorials_code_2_hyperband_checkpoint.py`)

experiment = get_experiment("hyperband-cifar10", storage=storage)
experiment.plot.partial_dependencies()

#%%
# You can select the hyperparameters to show with the argument ``params``.

experiment.plot.partial_dependencies(params=["gamma", "learning_rate"])

#%%
# Grid resolution
# ~~~~~~~~~~~~~~~
# The grid used for the partial dependency can be more or less coarse. Coarser grids
# will be faster to compute.
import time

experiment = get_experiment("2-dim-exp", storage=storage)
start = time.clock()
fig = experiment.plot.partial_dependencies(n_grid_points=5)
print(time.clock() - start, "seconds to compute")
fig

#%%
# With more points the grid is finer.
start = time.clock()
fig = experiment.plot.partial_dependencies(n_grid_points=50)
print(time.clock() - start, "seconds to compute")
fig

#%%
# Dependency approximation
# ~~~~~~~~~~~~~~~~~~~~~~~~
# By default, the hyperparameters are marginalized over 50 points. This may be suitable
# for a small 2-D search space but likely unsufficient for 5 dimensions or more.
# Here is an example with only 5 samples.

start = time.clock()
fig = experiment.plot.partial_dependencies(n_samples=5)
print(time.clock() - start, "seconds to compute")
fig

#%%
# And now with 200 samples.

start = time.clock()
fig = experiment.plot.partial_dependencies(n_samples=200)
print(time.clock() - start, "seconds to compute")
fig

#%%
# Special cases
# -------------
#
# Logarithmic scale
# ~~~~~~~~~~~~~~~~~
#
# Dimensions with a logarithmic prior :ref:`search-space-prior-loguniform` are linearized before
# being passed to the regression model (using log(dim) instead of dim directly). This means the
# model is trained and will be making predictions in the linearized space. The data is presented in
# logarithmic scale to the user, with the axis adjusted to log scale as well.
#
# Fidelity
# ~~~~~~~~
#
# The fidelity is considered as a logarithmic scale as well. See above for more information on how
# it is handled.
#
# Dimension with shape
# ~~~~~~~~~~~~~~~~~~~~
#
# If some dimensions have a :ref:`search-space-shape` larger than 1, they will be flattened so that
# each subdimension can be represented separately in the subplots.

# Load the data for the specified experiment
experiment = get_experiment("2-dim-shape-exp", storage=storage)
experiment.plot.partial_dependencies()

#%%
# In the example above, the dimension ``learning_rate~loguniform(1e-5, 1e-2, shape=3)``
# is flattened and represented with ``learning_rate[i]``. If the shape would be or more dimensions
# (ex: ``(3, 2)``), the indices would be ``learning_rate[i,j]`` with i=0..2 and j=0..1.

#%%
# The flattened hyperparameters can be fully selected with ``params=['<name>']``.
#

experiment.plot.partial_dependencies(params=["/learning_rate"])

#%%
# Or a subset of the flattened hyperparameters can be selected with ``params=['<name>[index]']``.
#

experiment.plot.partial_dependencies(params=["/learning_rate[0]", "/learning_rate[1]"])


#%%
# Categorical dimension
# ~~~~~~~~~~~~~~~~~~~~~
#
# Categorical dimensions are converted into integer values, so that the regression model
# can handle them. The integers are simply indices that are assigned to each category in arbitrary
# order. Here is an example where dimension ``mt-join`` has the prior
# ``choices(['mean', 'max', 'concat'])``.

# Load the data for the specified experiment
experiment = get_experiment("3-dim-cat-shape-exp", storage=storage)
experiment.plot.partial_dependencies(params=["/learning_rate[0]", "/mt-join"])

#%%
# .. Note::
#
#     For now categorical are plotted using a scatter plot, but a heat-map
#     would be more readable. Contributions are more than welcome! :)

#%%
# Finally we save the image to serve as a thumbnail for this example. See
# the guide
# :ref:`How to save <sphx_glr_auto_examples_how-tos_code_2_how_to_save.py>`
# for more information on image saving.

fig.write_image("../../docs/src/_static/par_dep_thumbnail.png")

# sphinx_gallery_thumbnail_path = '_static/par_dep_thumbnail.png'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Module running the init_only command
====================================

Creates a new experiment.

"""

import logging

import orion.core
import orion.core.io.experiment_builder as experiment_builder
from orion.core.cli import base as cli
from orion.core.cli import evc as evc_cli

log = logging.getLogger(__name__)
DESCRIPTION = "(DEPRECATED) Use command `orion hunt --init_only` instead"


def add_subparser(parser):
    """Return the parser that needs to be used for this command"""
    init_only_parser = parser.add_parser(
        "init_only", help=DESCRIPTION, description=DESCRIPTION
    )

    orion_group = cli.get_basic_args_group(
        init_only_parser, group_name="init_only arguments", group_help=""
    )

    orion.core.config.experiment.add_arguments(
        orion_group,
        rename=dict(max_broken="--exp-max-broken", max_trials="--exp-max-trials"),
    )

    orion_group.add_argument(
        "--max-trials",
        type=int,
        metavar="#",
        help="(DEPRECATED) This argument will be removed in v0.3. Use --exp-max-trials instead",
    )

    evc_cli.get_branching_args_group(init_only_parser)

    cli.get_user_args_group(init_only_parser)

    init_only_parser.set_defaults(func=main)
    init_only_parser.set_defaults(help_empty=True)  # Print help if command is empty

    return init_only_parser


def main(args):
    """Build and initialize experiment"""
    # By building the experiment, we create a new experiment document in database
    log.warning(
        "Command init_only is deprecated and will be removed in v0.3. "
        "Use orion hunt --init-only instead."
    )
    experiment_builder.build_from_args(args)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Perform a functional test of the setup command."""
import builtins
import os

import yaml

import orion.core
import orion.core.cli
from orion.core.io.database import Database


class _mock_input:
    """Mock `input` to return a serie of strings."""

    def __init__(self, values):
        """Set the serie of values"""
        self.values = values

    def __call__(self, *args):
        """Pop one value at each"""
        return self.values.pop(0)


def test_creation_when_not_existing(monkeypatch, tmp_path):
    """Test if a configuration file is created when it does not exist."""
    config_path = str(tmp_path) + "/tmp_config.yaml"
    monkeypatch.setattr(orion.core, "DEF_CONFIG_FILES_PATHS", [config_path])
    monkeypatch.setattr(builtins, "input", _mock_input(["mongodb", "host", "name"]))

    try:
        os.remove(config_path)
    except FileNotFoundError:
        pass

    orion.core.cli.main(["db", "setup"])

    assert os.path.exists(config_path)

    with open(config_path, "r") as output:
        content = yaml.safe_load(output)

    assert content == {"database": {"type": "mongodb", "name": "name", "host": "host"}}


def test_creation_when_exists(monkeypatch, tmp_path):
    """Test if the configuration file is overwritten when it exists."""
    config_path = str(tmp_path) + "/tmp_config.yaml"
    monkeypatch.setattr(orion.core, "DEF_CONFIG_FILES_PATHS", [config_path])
    monkeypatch.setattr(
        builtins, "input", _mock_input(["y", "mongodb", "host", "name"])
    )

    dump = {"database": {"type": "allo2", "name": "allo2", "host": "allo2"}}

    with open(config_path, "w") as output:
        yaml.dump(dump, output)

    orion.core.cli.main(["db", "setup"])

    with open(config_path, "r") as output:
        content = yaml.safe_load(output)

    assert content != dump


def test_stop_creation_when_exists(monkeypatch, tmp_path):
    """Test if the configuration file is overwritten when it exists."""
    config_path = str(tmp_path) + "/tmp_config.yaml"
    monkeypatch.setattr(orion.core, "DEF_CONFIG_FILES_PATHS", [config_path])
    monkeypatch.setattr(builtins, "input", _mock_input(["n"]))

    dump = {"database": {"type": "allo2", "name": "allo2", "host": "allo2"}}

    with open(config_path, "w") as output:
        yaml.dump(dump, output)

    orion.core.cli.main(["db", "setup"])

    with open(config_path, "r") as output:
        content = yaml.safe_load(output)

    assert content == dump


def test_invalid_database(monkeypatch, tmp_path, capsys):
    """Test if command prompt loops when invalid database is typed."""
    invalid_db_names = [
        "invalid database",
        "invalid database again",
        "2383ejdd",
        "another invalid database",
    ]
    config_path = str(tmp_path) + "/tmp_config.yaml"
    monkeypatch.setattr(orion.core, "DEF_CONFIG_FILES_PATHS", [config_path])
    monkeypatch.setattr(
        builtins,
        "input",
        _mock_input(
            [
                *invalid_db_names,
                "mongodb",
                "the host",
                "the name",
            ]
        ),
    )

    orion.core.cli.main(["db", "setup"])

    with open(config_path, "r") as output:
        content = yaml.safe_load(output)

    assert content == {
        "database": {"type": "mongodb", "name": "the name", "host": "the host"}
    }

    captured_output = capsys.readouterr().out
    for invalid_db_name in invalid_db_names:
        assert (
            "Unexpected value: {}. Must be one of: {}\n".format(
                invalid_db_name, ", ".join(sorted(Database.types.keys()))
            )
            in captured_output
        )


def test_defaults(monkeypatch, tmp_path):
    """Test if the default values are used when nothing user enters nothing."""
    config_path = str(tmp_path) + "/tmp_config.yaml"
    monkeypatch.setattr(orion.core, "DEF_CONFIG_FILES_PATHS", [config_path])
    monkeypatch.setattr(builtins, "input", _mock_input(["", "", ""]))

    orion.core.cli.main(["db", "setup"])

    with open(config_path, "r") as output:
        content = yaml.safe_load(output)

    assert content == {
        "database": {"type": "mongodb", "name": "orion", "host": "localhost"}
    }


def test_ephemeraldb(monkeypatch, tmp_path):
    """Test if config content is written for an ephemeraldb."""
    config_path = str(tmp_path) + "/tmp_config.yaml"
    monkeypatch.setattr(orion.core, "DEF_CONFIG_FILES_PATHS", [config_path])
    monkeypatch.setattr(builtins, "input", _mock_input(["ephemeraldb"]))

    orion.core.cli.main(["db", "setup"])

    with open(config_path, "r") as output:
        content = yaml.safe_load(output)

    assert content == {"database": {"type": "ephemeraldb"}}


def test_pickleddb(monkeypatch, tmp_path):
    """Test if config content is written for an pickleddb."""
    host = "my_pickles.db"

    config_path = str(tmp_path) + "/tmp_config.yaml"
    monkeypatch.setattr(orion.core, "DEF_CONFIG_FILES_PATHS", [config_path])
    monkeypatch.setattr(builtins, "input", _mock_input(["pickleddb", host]))

    orion.core.cli.main(["db", "setup"])

    with open(config_path, "r") as output:
        content = yaml.safe_load(output)

    assert content == {"database": {"type": "pickleddb", "host": host}}



# -*- coding: utf-8 -*-
"""Perform functional tests for the REST endpoint `/experiments`"""
import copy
import datetime

from orion.core.worker.trial import Trial
from orion.storage.base import get_storage

current_id = 0

base_experiment = dict(
    name="experiment-name",
    space={"x": "uniform(0, 200)"},
    metadata={
        "user": "test-user",
        "orion_version": "x.y.z",
        "datetime": datetime.datetime(1, 1, 1),
        "VCS": {
            "type": "git",
            "is_dirty": False,
            "HEAD_sha": "test",
            "active_branch": None,
            "diff_sha": "diff",
        },
    },
    version=1,
    pool_size=1,
    max_trials=10,
    max_broken=7,
    working_dir="",
    algorithms={"random": {"seed": 1}},
    producer={"strategy": "NoParallelStrategy"},
)

base_trial = {
    "experiment": None,
    "status": "new",
    "worker": None,
    "submit_time": datetime.datetime(1, 1, 1),
    "start_time": datetime.datetime(1, 1, 1, second=10),
    "end_time": datetime.datetime(1, 1, 2),
    "heartbeat": None,
    "results": [
        {"name": "loss", "type": "objective", "value": 0.05},
        {"name": "a", "type": "statistic", "value": 10},
        {"name": "b", "type": "statistic", "value": 5},
    ],
    "params": [
        {"name": "x", "type": "real", "value": 10.0},
    ],
}


class TestCollection:
    """Tests the server's response on experiments/"""

    def test_no_experiments(self, client):
        """Tests that the API returns a positive response when no experiments are present"""
        response = client.simulate_get("/experiments")

        assert response.json == []
        assert response.status == "200 OK"

    def test_send_name_and_versions(self, client):
        """Tests that the API returns all the experiments with their name and version"""
        expected = [{"name": "a", "version": 1}, {"name": "b", "version": 1}]

        _add_experiment(name="a", version=1, _id=1)
        _add_experiment(name="b", version=1, _id=2)

        response = client.simulate_get("/experiments")

        assert response.json == expected
        assert response.status == "200 OK"

    def test_latest_versions(self, client):
        """Tests that the API return the latest versions of each experiment"""
        expected = [{"name": "a", "version": 3}, {"name": "b", "version": 1}]

        _add_experiment(name="a", version=1, _id=1)
        _add_experiment(name="a", version=3, _id=2)
        _add_experiment(name="a", version=2, _id=3)
        _add_experiment(name="b", version=1, _id=4)

        response = client.simulate_get("/experiments")

        assert response.json == expected
        assert response.status == "200 OK"


class TestItem:
    """Tests the server's response to experiments/:name"""

    def test_non_existent_experiment(self, client):
        """
        Tests that a 404 response is returned when the experiment
        doesn't exist in the database
        """
        response = client.simulate_get("/experiments/a")

        assert response.status == "404 Not Found"
        assert response.json == {
            "title": "Experiment not found",
            "description": 'Experiment "a" does not exist',
        }

        _add_experiment(name="a", version=1, _id=1)
        response = client.simulate_get("/experiments/b")

        assert response.status == "404 Not Found"
        assert response.json == {
            "title": "Experiment not found",
            "description": 'Experiment "b" does not exist',
        }

    def test_experiment_specification(self, client):
        """Tests that the experiment returned is following the specification"""
        _add_experiment(name="a", version=1, _id=1)
        _add_trial(experiment=1, id_override="ae8", status="completed")

        response = client.simulate_get("/experiments/a")

        assert response.status == "200 OK"

        assert response.json["name"] == "a"
        assert response.json["version"] == 1
        assert response.json["status"] == "not done"
        assert response.json["trialsCompleted"] == 1
        assert response.json["startTime"] == "0001-01-01 00:00:00"  # TODO
        assert response.json["endTime"] == "0001-01-02 00:00:00"  # TODO
        assert len(response.json["user"])
        assert response.json["orionVersion"] == "x.y.z"

        _assert_config(response.json["config"])
        _assert_best_trial(response.json["bestTrial"])

    def test_default_is_latest_version(self, client):
        """Tests that the latest experiment is returned when no version parameter exists"""
        _add_experiment(name="a", version=1, _id=1)
        _add_experiment(name="a", version=3, _id=2)
        _add_experiment(name="a", version=2, _id=3)

        response = client.simulate_get("/experiments/a")

        assert response.status == "200 OK"
        assert response.json["version"] == 3

    def test_specific_version(self, client):
        """Tests that the specified version of an experiment is returned"""
        _add_experiment(name="a", version=1, _id=1)
        _add_experiment(name="a", version=2, _id=2)
        _add_experiment(name="a", version=3, _id=3)

        response = client.simulate_get("/experiments/a?version=2")

        assert response.status == "200 OK"
        assert response.json["version"] == 2

    def test_unknown_parameter(self, client):
        """
        Tests that if an unknown parameter is specified in
        the query string, an error is returned even if the experiment doesn't exist.
        """
        response = client.simulate_get("/experiments/a?unknown=true")

        assert response.status == "400 Bad Request"
        assert response.json == {
            "title": "Invalid parameter",
            "description": 'Parameter "unknown" is not supported. Expected parameter "version".',
        }

        _add_experiment(name="a", version=1, _id=1)

        response = client.simulate_get("/experiments/a?unknown=true")

        assert response.status == "400 Bad Request"
        assert response.json == {
            "title": "Invalid parameter",
            "description": 'Parameter "unknown" is not supported. Expected parameter "version".',
        }


def _add_experiment(**kwargs):
    """Adds experiment to the dummy orion instance"""
    base_experiment.update(copy.deepcopy(kwargs))
    get_storage().create_experiment(base_experiment)


def _add_trial(**kwargs):
    """Add trials to the dummy orion instance"""
    base_trial.update(copy.deepcopy(kwargs))
    get_storage().register_trial(Trial(**base_trial))


def _assert_config(config):
    """Asserts properties of the ``config`` dictionary"""
    assert config["poolSize"] == 1
    assert config["maxTrials"] == 10
    assert config["maxBroken"] == 7

    algorithm = config["algorithm"]
    assert algorithm["name"] == "random"
    assert algorithm["seed"] == 1

    space = config["space"]
    assert len(space) == 1
    assert space["x"] == "uniform(0, 200)"


def _assert_best_trial(best_trial):
    """Verifies properties of the best trial"""
    assert best_trial["id"] == "ae8"
    assert best_trial["submitTime"] == "0001-01-01 00:00:00"
    assert best_trial["startTime"] == "0001-01-01 00:00:10"
    assert best_trial["endTime"] == "0001-01-02 00:00:00"

    parameters = best_trial["parameters"]
    assert len(parameters) == 1
    assert parameters["x"] == 10.0

    assert best_trial["objective"] == 0.05

    statistics = best_trial["statistics"]
    assert statistics["a"] == 10
    assert statistics["b"] == 5



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Perform a functional test of the plot command."""
import os
import random

import pytest

import orion.core.cli
import orion.plotting.backend_plotly
from orion.core.cli.plot import IMAGE_TYPES, VALID_TYPES
from orion.plotting.base import SINGLE_EXPERIMENT_PLOTS
from orion.storage.base import get_storage
from orion.testing import AssertNewFile


def test_no_args(capsys):
    """Test that help is printed when no args are given."""
    with pytest.raises(SystemExit):
        orion.core.cli.main(["plot"])

    captured = capsys.readouterr().err

    assert "usage:" in captured
    assert "the following arguments are required: kind" in captured
    assert "Traceback" not in captured


def test_no_name(capsys):
    """Try to run the command without providing an experiment name"""
    returncode = orion.core.cli.main(["plot", "lpi"])
    assert returncode == 1

    captured = capsys.readouterr().err

    assert captured == "Error: No name provided for the experiment.\n"


def test_no_experiments(capsys, storage):
    """Test plot error message if experiment not found."""
    returncode = orion.core.cli.main(["plot", "regret", "--name", "idontexist"])
    assert returncode == 1

    captured = capsys.readouterr().err

    assert captured.startswith(
        "Error: No experiment with given name 'idontexist' and version '*'"
    )


def test_invalid_kind(capsys, single_with_trials):
    """Test that invalid kind leads to correct error message"""
    with pytest.raises(SystemExit):
        returncode = orion.core.cli.main(["plot", "blorg"])

    captured = capsys.readouterr().err

    assert "argument kind: invalid choice: 'blorg' (choose from 'lpi'," in captured


def test_default_type(single_with_trials):
    """Test that default type of image is used properly"""
    filename = "test_single_exp-v1_regret.png"
    with AssertNewFile(filename):
        returncode = orion.core.cli.main(
            ["plot", "regret", "--name", "test_single_exp"]
        )
        assert returncode == 0


def test_invalid_type(capsys, single_with_trials):
    """Test that invalid type leads to correct error message"""
    with pytest.raises(SystemExit):
        returncode = orion.core.cli.main(
            ["plot", "regret", "--name", "test_single_exp", "--type", "boom"]
        )

    captured = capsys.readouterr().err

    assert "-t/--type: invalid choice: 'boom' (choose from 'png'" in captured


@pytest.mark.parametrize("out_type", VALID_TYPES)
def test_type(out_type, single_with_trials):
    """Test that given type of image is used properly"""

    filename = f"test_single_exp-v1_regret.{out_type}"
    with AssertNewFile(filename):
        returncode = orion.core.cli.main(
            ["plot", "regret", "--name", "test_single_exp", "--type", out_type]
        )
        assert returncode == 0


def test_output(single_with_trials):
    """Test that output is used properly"""

    output = "custom"
    filename = f"{output}.png"
    with AssertNewFile(filename):
        returncode = orion.core.cli.main(
            ["plot", "regret", "--name", "test_single_exp", "--output", output]
        )
        assert returncode == 0


def test_output_type(single_with_trials):
    """Test that output is used properly with given type"""

    for output in ["custom", "custom.with.points", "custombadjpg"]:
        filename = f"{output}.jpg"
        with AssertNewFile(filename):
            returncode = orion.core.cli.main(
                [
                    "plot",
                    "regret",
                    "--name",
                    "test_single_exp",
                    "--output",
                    output,
                    "--type",
                    "jpg",
                ]
            )
            assert returncode == 0


def test_output_override_type(single_with_trials):
    """Test that output with type will override --type"""

    output = "custom.jpg"
    filename = output
    with AssertNewFile(filename):
        returncode = orion.core.cli.main(
            [
                "plot",
                "regret",
                "--name",
                "test_single_exp",
                "--output",
                output,
                "--type",
                "pdf",
            ]
        )
        assert returncode == 0


@pytest.mark.parametrize("kind", SINGLE_EXPERIMENT_PLOTS.keys())
def test_output_default_kind(monkeypatch, single_with_trials, kind):
    """Test that output default is used properly and kind is called"""

    plotting_function = getattr(orion.plotting.backend_plotly, kind)

    def mock_plot(*args, **kwargs):
        mock_plot.called = True
        return plotting_function(*args, **kwargs)

    mock_plot.called = False

    monkeypatch.setattr(f"orion.plotting.backend_plotly.{kind}", mock_plot)

    assert len(get_storage().fetch_trials(uid=single_with_trials["_id"])) > 0

    filename = f"test_single_exp-v1_{kind}.png"
    with AssertNewFile(filename):
        returncode = orion.core.cli.main(
            [
                "plot",
                kind,
                "--name",
                "test_single_exp",
            ]
        )
        assert returncode == 0

    assert mock_plot.called


def test_scale(monkeypatch, single_with_trials):
    """Test that scale is passed properly"""

    SCALE = random.uniform(0, 3)

    def check_args(self, output, scale, **kwargs):
        assert scale == SCALE

    monkeypatch.setattr("plotly.graph_objs._figure.Figure.write_image", check_args)

    returncode = orion.core.cli.main(
        ["plot", "regret", "--name", "test_single_exp", "--scale", str(SCALE)]
    )
    assert returncode == 0


@pytest.mark.parametrize("kind", SINGLE_EXPERIMENT_PLOTS.keys())
def test_no_trials(one_experiment, kind):
    """Test plotting works with empty experiments"""

    assert get_storage().fetch_trials(uid=one_experiment["_id"]) == []

    filename = f"test_single_exp-v1_{kind}.png"
    with AssertNewFile(filename):
        returncode = orion.core.cli.main(
            [
                "plot",
                kind,
                "--name",
                "test_single_exp",
            ]
        )
        assert returncode == 0


def test_html(monkeypatch, single_with_trials):
    """Test html is saved properly"""

    def mock_html(*args, **kwargs):
        mock_html.called = True
        return ""

    mock_html.called = False

    monkeypatch.setattr("plotly.graph_objs._figure.Figure.write_html", mock_html)

    returncode = orion.core.cli.main(
        ["plot", "regret", "--name", "test_single_exp", "--type", "html"]
    )
    assert returncode == 0

    assert mock_html.called


def test_json(monkeypatch, single_with_trials):
    """Test json is saved properly"""

    def mock_json(*args, **kwargs):
        mock_json.called = True
        return ""

    mock_json.called = False

    monkeypatch.setattr("plotly.graph_objs._figure.Figure.to_json", mock_json)

    returncode = orion.core.cli.main(
        ["plot", "regret", "--name", "test_single_exp", "--type", "json"]
    )
    assert returncode == 0

    assert mock_json.called


@pytest.mark.parametrize("out_type", IMAGE_TYPES)
def test_write_image(monkeypatch, single_with_trials, out_type):
    """Test all types for images use write_image"""

    def mock_write_image(*args, **kwargs):
        mock_write_image.called = True
        return ""

    mock_write_image.called = False

    monkeypatch.setattr(
        "plotly.graph_objs._figure.Figure.write_image", mock_write_image
    )

    returncode = orion.core.cli.main(
        ["plot", "regret", "--name", "test_single_exp", "--type", out_type]
    )
    assert returncode == 0

    assert mock_write_image.called



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for :mod:`orion.benchmark.task`."""

from orion.benchmark.task import Branin, CarromTable, EggHolder, RosenBrock


class TestBranin:
    """Test benchmark task branin"""

    def test_creation(self):
        """Test creation"""
        branin = Branin(2)
        assert branin.max_trials == 2
        assert branin.configuration == {"Branin": {"max_trials": 2}}

    def test_call(self):
        """Test to get task function"""
        branin = Branin(2)

        assert callable(branin)

        objectives = branin([1, 2])
        assert type(objectives[0]) == dict

    def test_search_space(self):
        """Test to get task search space"""
        branin = Branin(2)

        assert branin.get_search_space() == {
            "x": "uniform(0, 1, shape=2, precision=10)"
        }


class TestCarromTable:
    """Test benchmark task CarromTable"""

    def test_creation(self):
        """Test creation"""
        branin = CarromTable(2)
        assert branin.max_trials == 2
        assert branin.configuration == {"CarromTable": {"max_trials": 2}}

    def test_call(self):
        """Test to get task function"""
        carrom = CarromTable(2)

        assert callable(carrom)

        objectives = carrom([1, 2])
        assert type(objectives[0]) == dict

    def test_search_space(self):
        """Test to get task search space"""
        carrom = CarromTable(2)

        assert carrom.get_search_space() == {"x": "uniform(-10, 10, shape=2)"}


class TestEggHolder:
    """Test benchmark task EggHolder"""

    def test_creation(self):
        """Test creation"""
        branin = EggHolder(max_trials=2, dim=3)
        assert branin.max_trials == 2
        assert branin.configuration == {"EggHolder": {"dim": 3, "max_trials": 2}}

    def test_call(self):
        """Test to get task function"""
        egg = EggHolder(2)

        assert callable(egg)

        objectives = egg([1, 2])
        assert type(objectives[0]) == dict

    def test_search_space(self):
        """Test to get task search space"""
        egg = EggHolder(2)

        assert egg.get_search_space() == {"x": "uniform(-512, 512, shape=2)"}


class TestRosenBrock:
    """Test benchmark task RosenBrock"""

    def test_creation(self):
        """Test creation"""
        branin = RosenBrock(max_trials=2, dim=3)
        assert branin.max_trials == 2
        assert branin.configuration == {"RosenBrock": {"dim": 3, "max_trials": 2}}

    def test_call(self):
        """Test to get task function"""
        rb = RosenBrock(2)

        assert callable(rb)

        objectives = rb([1, 2])
        assert type(objectives[0]) == dict

    def test_search_space(self):
        """Test to get task search space"""
        rb = RosenBrock(2)
        assert rb.get_search_space() == {"x": "uniform(-5, 10, shape=2)"}



# -*- coding: utf-8 -*-
"""
Grid Search
===========
"""
import itertools
import logging

import numpy

from orion.algo.base import BaseAlgorithm
from orion.algo.space import Categorical, Fidelity, Integer, Real

log = logging.getLogger(__name__)


def grid(dim, num):
    """Build a one-dim grid of num points"""

    if dim.type == "categorical":
        return categorical_grid(dim, num)
    elif dim.type == "integer":
        return discrete_grid(dim, num)
    elif dim.type == "real":
        return real_grid(dim, num)
    elif dim.type == "fidelity":
        return fidelity_grid(dim, num)
    else:
        raise TypeError(
            "Grid Search only supports `real`, `integer`, `categorical` and `fidelity`: "
            f"`{dim.type}`\n"
            "For more information on dimension types, see "
            "https://orion.readthedocs.io/en/stable/user/searchspace.html"
        )


def fidelity_grid(dim, num):
    """Build fidelity grid, that is, only top value"""
    return [dim.interval()[1]]


def categorical_grid(dim, num):
    """Build categorical grid, that is, all categories"""
    categories = dim.interval()
    if len(categories) != num:
        log.warning(
            f"Categorical dimension {dim.name} does not have {num} choices: {categories}. "
            "Will use {len(categories)} choices instead."
        )
    return categories


def discrete_grid(dim, num):
    """Build discretized real grid"""
    grid = real_grid(dim, num)

    _, b = dim.interval()

    discrete_grid = [int(numpy.round(grid[0]))]
    for v in grid[1:]:
        int_v = int(numpy.round(v))
        if int_v <= discrete_grid[-1]:
            int_v = discrete_grid[-1] + 1

        if int_v > b:
            log.warning(
                f"Cannot list {num} discrete values for {dim.name}. "
                "Will use {len(discrete_grid)} points instead."
            )
            break

        discrete_grid.append(int_v)

    return discrete_grid


def real_grid(dim, num):
    """Build real grid"""
    if dim.prior_name.endswith("reciprocal"):
        a, b = dim.interval()
        return list(_log_grid(a, b, num))
    elif dim.prior_name.endswith("uniform"):
        a, b = dim.interval()
        return list(_lin_grid(a, b, num))
    else:
        raise TypeError(
            "Grid Search only supports `loguniform`, `uniform` and `choices`: "
            "`{}`".format(dim.prior_name)
        )


def _log_grid(a, b, num):
    return numpy.exp(_lin_grid(numpy.log(a), numpy.log(b), num))


def _lin_grid(a, b, num):
    return numpy.linspace(a, b, num=num)


class GridSearch(BaseAlgorithm):
    """Grid Search algorithm

    Parameters
    ----------
    n_values: int or dict
        Number of points for each dimensions, or dictionary specifying number of points for each
        dimension independently (name, n_values). For categorical dimensions, n_values will not be
        used, and all categories will be used to build the grid.
    """

    requires_type = None
    requires_dist = None
    requires_shape = "flattened"

    def __init__(self, space, n_values=100, seed=None):
        super(GridSearch, self).__init__(space, n_values=n_values, seed=seed)
        self.n = 0
        self.grid = None

    def _initialize(self):
        """Initialize the grid once the space is transformed"""
        n_values = self.n_values
        if not isinstance(n_values, dict):
            n_values = {name: self.n_values for name in self.space.keys()}

        self.grid = self.build_grid(
            self.space, n_values, getattr(self, "max_trials", 10000)
        )

    @staticmethod
    def build_grid(space, n_values, max_trials=10000):
        """Build a grid of trials

        Parameters
        ----------
        n_values: int or dict
            Number of points for each dimensions, or dictionary specifying number of points for each
            dimension independently (name, n_values). For categorical dimensions, n_values will not be
            used, and all categories will be used to build the grid.
        max_trials: int
            Maximum number of trials for the grid. If n_values lead to more trials than max_trials,
            the n_values will be adjusted down. Will raise ValueError if it is impossible to build
            a grid smaller than max_trials (for instance if choices are too large).

        """
        adjust = 0
        n_trials = float("inf")

        while n_trials > max_trials:
            coordinates = []
            capped_values = []
            for name, dim in space.items():
                capped_value = max(n_values[name] - adjust, 1)
                capped_values.append(capped_value)
                coordinates.append(list(grid(dim, capped_value)))

            if all(value <= 1 for value in capped_values):
                raise ValueError(
                    f"Cannot build a grid smaller than {max_trials}. "
                    "Try reducing the number of choices in categorical dimensions."
                )

            n_trials = numpy.prod([len(dim_values) for dim_values in coordinates])
            # TODO: Use binary search instead of incrementing by one.
            adjust += 1

        if adjust > 1:
            log.warning(
                f"`n_values` reduced by {adjust-1} to limit number of trials below {max_trials}."
            )

        return list(itertools.product(*coordinates))

    @property
    def state_dict(self):
        """Return a state dict that can be used to reset the state of the algorithm."""
        state_dict = super(GridSearch, self).state_dict
        state_dict["grid"] = self.grid
        return state_dict

    def set_state(self, state_dict):
        """Reset the state of the algorithm based on the given state_dict

        Parameters
        ----------
        state_dict: dict
            Dictionary representing state of an algorithm
        """
        super(GridSearch, self).set_state(state_dict)
        self.grid = state_dict["grid"]

    def suggest(self, num):
        """Return the entire grid of suggestions

        Returns
        -------
        list of points or None
            A list of lists representing points suggested by the algorithm. The algorithm may opt
            out if it cannot make a good suggestion at the moment (it may be waiting for other
            trials to complete), in which case it will return None.

        """
        if self.grid is None:
            self._initialize()
        i = 0
        points = []
        while len(points) < num and i < len(self.grid):
            point = self.grid[i]
            if not self.has_suggested(point):
                self.register(point)
                points.append(point)
            i += 1

        return points

    @property
    def is_done(self):
        """Return True when all grid has been covered."""
        return (
            super(GridSearch, self).is_done
            or self.grid is not None
            and self.n_suggested >= len(self.grid)
        )

    @property
    def configuration(self):
        """Return tunable elements of this algorithm in a dictionary form
        appropriate for saving.

        """
        # NOTE: Override parent method to ignore `seed`
        return {self.__class__.__name__.lower(): {"n_values": self.n_values}}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Collection of tests for :mod:`orion.core.worker.strategies`."""
import logging

import pytest

from orion.core.worker.strategy import (
    MaxParallelStrategy,
    MeanParallelStrategy,
    NoParallelStrategy,
    Strategy,
    StubParallelStrategy,
)
from orion.core.worker.trial import Trial


@pytest.fixture
def observations():
    """10 objective observations"""
    points = [i for i in range(10)]
    results = [{"objective": points[i]} for i in range(10)]

    return points, results


@pytest.fixture
def incomplete_trial():
    """Return a single trial without results"""
    return Trial(params=[{"name": "a", "type": "integer", "value": 6}])


@pytest.fixture
def corrupted_trial():
    """Return a corrupted trial with results but status reserved"""
    return Trial(
        params=[{"name": "a", "type": "integer", "value": 6}],
        results=[{"name": "objective", "type": "objective", "value": 1}],
        status="reserved",
    )


strategies = [
    "MaxParallelStrategy",
    "MeanParallelStrategy",
    "NoParallelStrategy",
    "StubParallelStrategy",
]


@pytest.mark.parametrize("strategy", strategies)
def test_handle_corrupted_trials(caplog, strategy, corrupted_trial):
    """Verify that corrupted trials are handled properly"""
    with caplog.at_level(logging.WARNING, logger="orion.core.worker.strategy"):
        lie = Strategy(strategy).lie(corrupted_trial)

    match = "Trial `{}` has an objective but status is not completed".format(
        corrupted_trial.id
    )
    assert match in caplog.text

    assert lie is not None
    assert lie.value == corrupted_trial.objective.value


@pytest.mark.parametrize("strategy", strategies)
def test_handle_uncorrupted_trials(caplog, strategy, incomplete_trial):
    """Verify that no warning is logged if trial is valid"""
    with caplog.at_level(logging.WARNING, logger="orion.core.worker.strategy"):
        Strategy(strategy).lie(incomplete_trial)

    assert "Trial `{}` has an objective but status is not completed" not in caplog.text


class TestStrategyFactory:
    """Test creating a parallel strategy with the Strategy class"""

    def test_create_noparallel(self):
        """Test creating a NoParallelStrategy class"""
        strategy = Strategy("NoParallelStrategy")
        assert isinstance(strategy, NoParallelStrategy)

    def test_create_meanparallel(self):
        """Test creating a MeanParallelStrategy class"""
        strategy = Strategy("MeanParallelStrategy")
        assert isinstance(strategy, MeanParallelStrategy)


class TestParallelStrategies:
    """Test the different parallel strategy methods"""

    def test_max_parallel_strategy(self, observations, incomplete_trial):
        """Test that MaxParallelStrategy lies using the max"""
        points, results = observations

        strategy = MaxParallelStrategy()
        strategy.observe(points, results)
        lying_result = strategy.lie(incomplete_trial)

        max_value = max(result["objective"] for result in results)
        assert lying_result.value == max_value

    def test_mean_parallel_strategy(self, observations, incomplete_trial):
        """Test that MeanParallelStrategy lies using the mean"""
        points, results = observations

        strategy = MeanParallelStrategy()
        strategy.observe(points, results)
        lying_result = strategy.lie(incomplete_trial)

        mean_value = sum(result["objective"] for result in results) / float(
            len(results)
        )
        assert lying_result.value == mean_value

    def test_no_parallel_strategy(self, observations, incomplete_trial):
        """Test that NoParallelStrategy lies outputs None"""
        points, results = observations

        strategy = NoParallelStrategy()
        strategy.observe(points, results)
        lying_result = strategy.lie(incomplete_trial)
        assert lying_result is None

    def test_stub_parallel_strategy(self, observations, incomplete_trial):
        """Test that NoParallelStrategy lies outputs None"""
        points, results = observations

        strategy = StubParallelStrategy()
        strategy.observe(points, results)
        lying_result = strategy.lie(incomplete_trial)
        assert lying_result.value is None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Example usage and tests for :mod:`orion.algo.base`."""

from orion.algo.base import BaseAlgorithm
from orion.algo.space import Integer, Real, Space


def test_init(dumbalgo):
    """Check if initialization works for nested algos."""
    nested_algo = {"DumbAlgo": dict(value=6, scoring=5)}
    algo = dumbalgo(8, value=1, subone=nested_algo)
    assert algo.space == 8
    assert algo.value == 1
    assert algo.scoring == 0
    assert algo.judgement is None
    assert algo.suspend is False
    assert algo.done is False
    assert isinstance(algo.subone, BaseAlgorithm)
    assert algo.subone.space == 8
    assert algo.subone.value == 6
    assert algo.subone.scoring == 5


def test_configuration(dumbalgo):
    """Check configuration getter works for nested algos."""
    nested_algo = {"DumbAlgo": dict(value=6, scoring=5)}
    algo = dumbalgo(8, value=1, subone=nested_algo)
    config = algo.configuration
    assert config == {
        "dumbalgo": {
            "seed": None,
            "value": 1,
            "scoring": 0,
            "judgement": None,
            "suspend": False,
            "done": False,
            "subone": {
                "dumbalgo": {
                    "seed": None,
                    "value": 6,
                    "scoring": 5,
                    "judgement": None,
                    "suspend": False,
                    "done": False,
                }
            },
        }
    }


def test_space_setter(dumbalgo):
    """Check whether space setter works for nested algos."""
    nested_algo = {
        "DumbAlgo": dict(
            value=9,
        )
    }
    nested_algo2 = {
        "DumbAlgo": dict(
            judgement=10,
        )
    }
    algo = dumbalgo(8, value=1, naedw=nested_algo, naekei=nested_algo2)
    algo.space = "etsh"
    assert algo.space == "etsh"
    assert algo.naedw.space == "etsh"
    assert algo.naedw.value == 9
    assert algo.naekei.space == "etsh"
    assert algo.naekei.judgement == 10


def test_state_dict(dumbalgo):
    """Check whether trials_info is in the state dict"""
    nested_algo = {"DumbAlgo": dict(value=6, scoring=5)}
    algo = dumbalgo(8, value=1, subone=nested_algo)
    algo.suggest(1)
    assert not algo.state_dict["_trials_info"]
    algo.observe([(1, 2)], [{"objective": 3}])
    assert len(algo.state_dict["_trials_info"]) == 1
    algo.observe([(1, 2)], [{"objective": 3}])
    assert len(algo.state_dict["_trials_info"]) == 1


def test_is_done_cardinality(monkeypatch, dumbalgo):
    """Check whether algorithm will stop with base algorithm cardinality check"""
    monkeypatch.delattr(dumbalgo, "is_done")

    space = Space()
    space.register(Integer("yolo1", "uniform", 1, 4))

    algo = dumbalgo(space)
    algo.suggest(6)
    for i in range(1, 6):
        algo.observe([[i]], [{"objective": 3}])

    assert len(algo.state_dict["_trials_info"]) == 5
    assert algo.is_done

    space = Space()
    space.register(Real("yolo1", "uniform", 1, 4))

    algo = dumbalgo(space)
    algo.suggest(6)
    for i in range(1, 6):
        algo.observe([[i]], [{"objective": 3}])

    assert len(algo.state_dict["_trials_info"]) == 5
    assert not algo.is_done


def test_is_done_max_trials(monkeypatch, dumbalgo):
    """Check whether algorithm will stop with base algorithm max_trials check"""
    monkeypatch.delattr(dumbalgo, "is_done")

    space = Space()
    space.register(Real("yolo1", "uniform", 1, 4))

    algo = dumbalgo(space)
    algo.suggest(5)
    for i in range(1, 5):
        algo.observe([[i]], [{"objective": 3}])

    assert len(algo.state_dict["_trials_info"]) == 4
    assert not algo.is_done

    dumbalgo.max_trials = 4
    assert algo.is_done



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for :mod:`orion.algo.evolution_es`."""

import copy
import hashlib

import numpy as np
import pytest

from orion.algo.evolution_es import BracketEVES, EvolutionES, compute_budgets
from orion.algo.space import Fidelity, Real, Space
from orion.testing.algo import BaseAlgoTests


@pytest.fixture
def space():
    """Create a Space with a real dimension and a fidelity value."""
    space = Space()
    space.register(Real("lr", "uniform", 0, 1))
    space.register(Fidelity("epoch", 1, 9, 1))
    return space


@pytest.fixture
def space1():
    """Create a Space with two real dimensions and a fidelity value."""
    space = Space()
    space.register(Real("lr", "uniform", 0, 1))
    space.register(Real("weight_decay", "uniform", 0, 1))
    space.register(Fidelity("epoch", 1, 8, 2))
    return space


@pytest.fixture
def space2():
    """Create a Space with two real dimensions."""
    space = Space()
    space.register(Real("lr", "uniform", 0, 1))
    space.register(Real("weight_decay", "uniform", 0, 1))
    return space


@pytest.fixture
def budgets():
    """Return a configuration for a bracket."""
    return [(30, 4), (30, 5), (30, 6)]


@pytest.fixture
def evolution(space1):
    """Return an instance of EvolutionES."""
    return EvolutionES(space1, repetitions=1, nums_population=4)


@pytest.fixture
def bracket(budgets, evolution, space1):
    """Return a `Bracket` instance configured with `b_config`."""
    return BracketEVES(evolution, budgets, 1)


@pytest.fixture
def evolution_customer_mutate(space1):
    """Return an instance of EvolutionES."""
    return EvolutionES(
        space1,
        repetitions=1,
        nums_population=4,
        mutate="orion.core.utils.tests.customized_mutate_example",
    )


@pytest.fixture
def rung_0():
    """Create fake points and objectives for rung 0."""
    points = np.linspace(0, 8, 9)
    return dict(
        n_trials=9,
        resources=1,
        results={
            hashlib.md5(str([point]).encode("utf-8")).hexdigest(): (point, (1, point))
            for point in points
        },
    )


@pytest.fixture
def rung_1(rung_0):
    """Create fake points and objectives for rung 1."""
    values = map(
        lambda v: (v[0], (3, v[0])), list(sorted(rung_0["results"].values()))[:3]
    )
    return dict(
        n_trials=3,
        resources=3,
        results={
            hashlib.md5(str([value[0]]).encode("utf-8")).hexdigest(): value
            for value in values
        },
    )


@pytest.fixture
def rung_2(rung_1):
    """Create fake points and objectives for rung 2."""
    values = map(
        lambda v: (v[0], (9, v[0])), list(sorted(rung_1["results"].values()))[:1]
    )
    return dict(
        n_trials=1,
        resources=9,
        results={
            hashlib.md5(str([value[0]]).encode("utf-8")).hexdigest(): value
            for value in values
        },
    )


@pytest.fixture
def rung_3():
    """Create fake points and objectives for rung 3."""
    points = np.linspace(1, 4, 4)
    return dict(
        n_trials=4,
        resources=1,
        results={
            hashlib.md5(str([point]).encode("utf-8")).hexdigest(): (
                point,
                (np.power(2, (point - 1)), 1.0 / point, 1.0 / (point * point)),
            )
            for point in points
        },
    )


@pytest.fixture
def rung_4():
    """Create duplicated fake points and objectives for rung 4."""
    points = np.linspace(1, 4, 4)
    return dict(
        n_trials=4,
        resources=1,
        results={
            hashlib.md5(str([point]).encode("utf-8")).hexdigest(): (
                point,
                (1, point // 2, point // 2),
            )
            for point in points
        },
    )


def test_compute_budgets():
    """Verify proper computation of budgets on a logarithmic scale"""
    # Check typical values
    assert compute_budgets(1, 3, 1, 2, 1) == [[(2, 1), (2, 2), (2, 3)]]
    assert compute_budgets(1, 4, 2, 4, 2) == [[(4, 1), (4, 2), (4, 4)]]


def test_customized_mutate_population(space1, rung_3, budgets):
    """Verify customized mutated candidates is generated correctly."""
    customerized_dict = {
        "function": "orion.testing.algo.customized_mutate_example",
        "multiply_factor": 2.0,
        "add_factor": 1,
    }
    population_range = len(rung_3["results"])
    algo = EvolutionES(
        space1,
        repetitions=1,
        nums_population=population_range,
        mutate=customerized_dict,
    )
    algo.brackets[0] = BracketEVES(algo, budgets, 1)

    red_team = [0, 2]
    blue_team = [1, 3]
    for i in range(population_range):
        for j in [1, 2]:
            algo.brackets[0].eves.population[j][i] = list(rung_3["results"].values())[
                i
            ][1][j]
        algo.brackets[0].eves.performance[i] = list(rung_3["results"].values())[i][0]

    org_data = np.stack(
        (
            list(algo.brackets[0].eves.population.values())[0],
            list(algo.brackets[0].eves.population.values())[1],
        ),
        axis=0,
    ).T

    org_data = copy.deepcopy(org_data)

    algo.brackets[0]._mutate_population(
        red_team, blue_team, rung_3["results"], population_range, fidelity=2
    )

    mutated_data = np.stack(
        (
            list(algo.brackets[0].eves.population.values())[0],
            list(algo.brackets[0].eves.population.values())[1],
        ),
        axis=0,
    ).T

    # Winner team will be [0, 2], so [0, 2] will be remained, [1, 3] will be mutated.
    assert org_data.shape == mutated_data.shape
    assert (mutated_data[0] == org_data[0]).all()
    assert (mutated_data[2] == org_data[2]).all()
    assert (mutated_data[1] != org_data[1]).any()
    assert (mutated_data[3] != org_data[3]).any()
    assert (mutated_data[1] != org_data[0]).any()
    assert (mutated_data[3] != org_data[2]).any()

    # For each individual, mutation occurs in only one dimension chosen from two.
    # Customized test mutation function is divided by 2 for real type.
    if mutated_data[1][0] == org_data[0][0] / customerized_dict["multiply_factor"]:
        assert mutated_data[1][1] == org_data[0][1]
    else:
        assert (
            mutated_data[1][1] == org_data[0][1] / customerized_dict["multiply_factor"]
        )

    if mutated_data[3][0] == org_data[2][0] / customerized_dict["multiply_factor"]:
        assert mutated_data[3][1] == org_data[2][1]
    else:
        assert (
            mutated_data[3][1] == org_data[2][1] / customerized_dict["multiply_factor"]
        )


class TestEvolutionES:
    """Tests for the algo Evolution."""

    def test_register(self, evolution, bracket, rung_0, rung_1):
        """Check that a point is registered inside the bracket."""
        evolution.brackets = [bracket]
        bracket.hyperband = evolution
        bracket.eves = evolution
        bracket.rungs = [rung_0, rung_1]
        point = (1, 0.0)
        point_hash = hashlib.md5(str([0.0]).encode("utf-8")).hexdigest()

        evolution.observe([point], [{"objective": 0.0}])

        assert len(bracket.rungs[0])
        assert point_hash in bracket.rungs[0]["results"]
        assert (0.0, point) == bracket.rungs[0]["results"][point_hash]


class TestBracketEVES:
    """Tests for `BracketEVES` class.."""

    def test_get_teams(self, bracket, rung_3):
        """Test that correct team is promoted."""
        bracket.rungs[0] = rung_3
        rung, population_range, red_team, blue_team = bracket._get_teams(0)
        assert len(list(rung.values())) == 4
        assert bracket.search_space_remove_fidelity == [1, 2]
        assert population_range == 4
        assert set(red_team).union(set(blue_team)) == {0, 1, 2, 3}
        assert set(red_team).intersection(set(blue_team)) == set()

    def test_mutate_population(self, bracket, rung_3):
        """Verify mutated candidates is generated correctly."""
        red_team = [0, 2]
        blue_team = [1, 3]
        population_range = 4
        for i in range(4):
            for j in [1, 2]:
                bracket.eves.population[j][i] = list(rung_3["results"].values())[i][1][
                    j
                ]
            bracket.eves.performance[i] = list(rung_3["results"].values())[i][0]

        org_data = np.stack(
            (
                list(bracket.eves.population.values())[0],
                list(bracket.eves.population.values())[1],
            ),
            axis=0,
        ).T

        org_data = copy.deepcopy(org_data)

        bracket._mutate_population(
            red_team, blue_team, rung_3["results"], population_range, fidelity=2
        )

        mutated_data = np.stack(
            (
                list(bracket.eves.population.values())[0],
                list(bracket.eves.population.values())[1],
            ),
            axis=0,
        ).T

        # Winner team will be [0, 2], so [0, 2] will be remained, [1, 3] will be mutated.
        assert org_data.shape == mutated_data.shape
        assert (mutated_data[0] == org_data[0]).all()
        assert (mutated_data[2] == org_data[2]).all()
        assert (mutated_data[1] != org_data[1]).any()
        assert (mutated_data[3] != org_data[3]).any()
        assert (mutated_data[1] != org_data[0]).any()
        assert (mutated_data[3] != org_data[2]).any()

        # For each individual, mutation occurs in only one dimension chosen from two.
        if mutated_data[1][0] != org_data[0][0]:
            assert mutated_data[1][1] == org_data[0][1]
        else:
            assert mutated_data[1][1] != org_data[0][1]

        if mutated_data[3][0] != org_data[2][0]:
            assert mutated_data[3][1] == org_data[2][1]
        else:
            assert mutated_data[3][1] != org_data[2][1]

    def test_duplicated_mutated_population(self, bracket, rung_4):
        """Verify duplicated candidates can be found and processed correctly."""
        red_team = [0, 2]
        blue_team = [0, 2]  # no mutate occur at first.
        population_range = 4
        for i in range(4):
            for j in [1, 2]:
                bracket.eves.population[j][i] = list(rung_4["results"].values())[i][1][
                    j
                ]
        points, nums_all_equal = bracket._mutate_population(
            red_team, blue_team, rung_4["results"], population_range, fidelity=2
        )

        # In this case, duplication will occur, and we can make it mutate one more time.
        # The points 1 and 2 should be different, while one of nums_all_equal should be 1.
        if points[1][1] != points[2][1]:
            assert points[1][2] == points[2][2]
        else:
            assert points[1][2] != points[2][2]

        assert nums_all_equal[0] == 0
        assert nums_all_equal[1] == 0
        assert nums_all_equal[2] == 1
        assert nums_all_equal[3] == 0

    def test_mutate_points(self, bracket, rung_3):
        """Test that correct point is promoted."""
        red_team = [0, 2]
        blue_team = [0, 2]
        population_range = 4
        for i in range(4):
            for j in [1, 2]:
                bracket.eves.population[j][i] = list(rung_3["results"].values())[i][1][
                    j
                ]
        points, nums_all_equal = bracket._mutate_population(
            red_team, blue_team, rung_3["results"], population_range, fidelity=2
        )
        assert points[0] == (2, 1.0, 1.0)
        assert points[1] == (2, 1.0 / 2, 1.0 / 4)
        assert (nums_all_equal == 0).all()


class TestGenericEvolutionES(BaseAlgoTests):
    algo_name = "evolutiones"
    config = {
        "seed": 123456,
        "repetitions": 3,
        "nums_population": 20,
        "max_retries": 1000,
        "mutate": None,
    }
    space = {"x": "uniform(0, 1)", "f": "fidelity(1, 10, base=2)"}

    @pytest.mark.skip(reason="See https://github.com/Epistimio/orion/issues/598")
    def test_is_done_cardinality(self):
        space = self.update_space(
            {
                # Increase fidelity to increase number of trials in first rungs
                "f": "fidelity(1, 100, base=2)",
                "x": "uniform(0, 4, discrete=True)",
                "y": "choices(['a', 'b', 'c'])",
                "z": "loguniform(1, 6, discrete=True)",
            }
        )
        space = self.create_space(space)
        assert space.cardinality == 5 * 3 * 6

        algo = self.create_algo(space=space)

        for rung in range(len(algo.algorithm.brackets[0].rungs)):
            assert not algo.is_done
            n_sampled = len(algo.algorithm.sampled)
            n_trials = len(algo.algorithm.trial_to_brackets)
            points = algo.suggest()
            if points is None:
                break
            assert len(algo.algorithm.sampled) == n_sampled + len(points)
            assert len(algo.algorithm.trial_to_brackets) == space.cardinality

            # We reached max number of trials we can suggest before observing any.
            assert algo.suggest() is None

            assert not algo.is_done

            for i, point in enumerate(points):
                algo.observe([point], [dict(objective=i)])

        assert algo.is_done

    @pytest.mark.parametrize("num", [100000, 1])
    def test_is_done_max_trials(self, num):
        space = self.create_space()

        MAX_TRIALS = 10
        algo = self.create_algo(space=space)
        algo.algorithm.max_trials = MAX_TRIALS

        objective = 0
        while not algo.is_done:
            points = algo.suggest(num)
            assert points
            if points:
                self.observe_points(points, algo, objective)
                objective += len(points)

        # Hyperband should ignore max trials.
        assert algo.n_observed > MAX_TRIALS
        assert algo.is_done

    @pytest.mark.skip(reason="See https://github.com/Epistimio/orion/issues/599")
    def test_optimize_branin(self):
        pass

    def infer_repetition_and_rung(self, num):
        budgets = list(np.cumsum(BUDGETS))
        if num >= budgets[-1] * 2:
            return 3, -1
        elif num >= budgets[-1]:
            return 2, -1

        if num <= 1:
            return 1, -1

        return 1, budgets.index(num)

    def assert_callbacks(self, spy, num, algo):

        if num == 0:
            return

        repetition_id, rung_id = self.infer_repetition_and_rung(num)

        brackets = algo.algorithm.brackets

        assert len(brackets) == repetition_id

        for j in range(0, rung_id + 1):
            for bracket in brackets:
                assert len(bracket.rungs[j]["results"]) > 0, (bracket, j)


BUDGETS = [20, 20, 20, 20]


TestGenericEvolutionES.set_phases(
    [("random", 0, "space.sample")]
    + [(f"rung{i}", budget, "suggest") for i, budget in enumerate(np.cumsum(BUDGETS))]
    + [("rep1-rung1", sum(BUDGETS), "suggest")]
    + [("rep2-rung1", sum(BUDGETS) * 2, "suggest")]
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Common fixtures and utils for io tests."""


import copy
import os

import pytest

from orion.core.evc import conflicts


@pytest.fixture()
def config_file():
    """Open config file with new config"""
    file_path = os.path.join(
        os.path.dirname(os.path.abspath(__file__)), "orion_config.yaml"
    )

    return open(file_path)


@pytest.fixture()
def old_config_file():
    """Open config file with original config from an experiment in db"""
    file_path = os.path.join(
        os.path.dirname(os.path.abspath(__file__)), "orion_old_config.yaml"
    )

    return open(file_path)


@pytest.fixture()
def incomplete_config_file():
    """Open config file with partial database configuration"""
    file_path = os.path.join(
        os.path.dirname(os.path.abspath(__file__)), "orion_incomplete_config.yaml"
    )

    return open(file_path)


@pytest.fixture
def parent_config():
    """Generate a new experiment configuration"""
    return dict(_id="test", name="test", metadata={"user": "corneauf"}, version=1)


@pytest.fixture
def child_config(parent_config):
    """Generate a new experiment configuration"""
    config = copy.deepcopy(parent_config)
    config["_id"] = "test2"
    config["refers"] = {"parent_id": "test"}
    config["version"] = 2

    return config


@pytest.fixture
def experiment_name_conflict(storage, parent_config, child_config):
    """Generate an experiment name conflict"""
    exps = storage.fetch_experiments({"name": "test"}) + storage.fetch_experiments(
        {"name": "test2"}
    )
    for exp in exps:
        storage.delete_experiment(uid=exp["_id"])
    storage.create_experiment(parent_config)
    storage.create_experiment(child_config)
    return conflicts.ExperimentNameConflict(parent_config, parent_config)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Perform functional tests for db rm."""
import zlib

import pytest

import orion.core.cli
from orion.storage.base import get_storage


def hsh(name, version):
    return zlib.adler32(str((name, version)).encode())


def execute(command, assert_code=0):
    """Execute orion command and return returncode"""
    returncode = orion.core.cli.main(command.split(" "))
    assert returncode == assert_code


def test_no_exp(setup_pickleddb_database, capsys):
    """Test that rm non-existing exp exits gracefully"""
    execute("db rm i-dont-exist", assert_code=1)

    captured = capsys.readouterr()

    assert captured.err.startswith(
        "Error: No experiment with given name 'i-dont-exist'"
    )


def test_confirm_name(monkeypatch, single_with_trials):
    """Test name must be confirmed for deletion"""

    def incorrect_name(*args):
        return "oops"

    monkeypatch.setattr("builtins.input", incorrect_name)

    with pytest.raises(SystemExit):
        execute("db rm test_single_exp")

    def correct_name(*args):
        return "test_single_exp"

    monkeypatch.setattr("builtins.input", correct_name)

    assert len(get_storage().fetch_experiments({})) == 1
    execute("db rm test_single_exp")
    assert len(get_storage().fetch_experiments({})) == 0


def test_one_exp(single_with_trials):
    """Test that one exp is deleted properly"""
    assert len(get_storage().fetch_experiments({})) == 1
    assert len(get_storage()._fetch_trials({})) > 0
    execute("db rm -f test_single_exp")
    assert len(get_storage().fetch_experiments({})) == 0
    assert len(get_storage()._fetch_trials({})) == 0


def test_rm_all_evc(three_family_branch_with_trials):
    """Test that deleting root removes all experiments"""
    assert len(get_storage().fetch_experiments({})) == 3
    assert len(get_storage()._fetch_trials({})) > 0
    execute("db rm -f test_double_exp --version 1")
    assert len(get_storage().fetch_experiments({})) == 0
    assert len(get_storage()._fetch_trials({})) == 0


def test_rm_under_evc(three_family_branch_with_trials):
    """Test that deleting an experiment removes all children"""
    assert len(get_storage().fetch_experiments({})) == 3
    assert len(get_storage()._fetch_trials({})) > 0
    execute("db rm -f test_double_exp_child --version 1")
    assert len(get_storage().fetch_experiments({})) == 1
    assert len(get_storage()._fetch_trials({})) > 0
    # TODO: Test that the correct trials were deleted


def test_rm_default_leaf(three_experiments_same_name):
    """Test that deleting an experiment removes the leaf by default"""
    assert len(get_storage().fetch_experiments({})) == 3
    execute("db rm -f test_single_exp")
    assert len(get_storage().fetch_experiments({})) == 2


def test_rm_trials_by_status(single_with_trials):
    """Test that trials can be deleted by status"""
    trials = get_storage()._fetch_trials({})
    n_broken = sum(trial.status == "broken" for trial in trials)
    assert n_broken > 0
    execute("db rm -f test_single_exp --status broken")
    assert len(get_storage()._fetch_trials({})) == len(trials) - n_broken


def test_rm_trials_all(single_with_trials):
    """Test that trials all be deleted with '*'"""
    assert len(get_storage()._fetch_trials({})) > 0
    execute("db rm -f test_single_exp --status *")
    assert len(get_storage()._fetch_trials({})) == 0


def test_rm_trials_in_evc(three_family_branch_with_trials):
    """Test that trials of parent experiment are not deleted"""
    assert len(get_storage().fetch_experiments({})) == 3
    assert (
        len(get_storage()._fetch_trials({"experiment": hsh("test_double_exp", 1)})) > 0
    )
    assert (
        len(
            get_storage()._fetch_trials({"experiment": hsh("test_double_exp_child", 1)})
        )
        > 0
    )
    assert (
        len(
            get_storage()._fetch_trials(
                {"experiment": hsh("test_double_exp_grand_child", 1)}
            )
        )
        > 0
    )
    execute("db rm -f test_double_exp_child --status *")
    # Make sure no experiments were deleted
    assert len(get_storage().fetch_experiments({})) == 3
    # Make sure only trials of given experiment were deleted
    assert (
        len(get_storage()._fetch_trials({"experiment": hsh("test_double_exp", 1)})) > 0
    )
    assert (
        len(
            get_storage()._fetch_trials({"experiment": hsh("test_double_exp_child", 1)})
        )
        == 0
    )
    assert (
        len(
            get_storage()._fetch_trials(
                {"experiment": hsh("test_double_exp_grand_child", 1)}
            )
        )
        == 0
    )
